
#include "basic_io.h"
#include "LCD.h"
#include "Test.h"

extern int Status;
extern int j;

void KEY0_Pressed() {
  char Text1[16] = "Playing...      ";
  char Text2[16] = "Stopped...      ";
    
  LCD_Line2();
  //  Show Text to LCD
  
  if (Status == 0) {
    Status = 1;
    LCD_Show_Text(Text1);
    IOWR(NIOS_CONTROL_SIGNALS_BASE, 0, 0x9);
    printf("Button 0 pressed. Playing...\n");      
  } else {
    Status = 0;
    LCD_Show_Text(Text2);    
    IOWR(NIOS_CONTROL_SIGNALS_BASE, 0, 0x8);    
    printf("Button 0 pressed. Stoping...\n");          
  }
}

void KEY1_Pressed() {
  char Text[16] = "Forwarded...    ";
  int switches;
  
  LCD_Line2();
  //  Show Text to LCD
  LCD_Show_Text(Text);
  
  // Fast forward
  switches = IORD(SWITCH_PIO_BASE, 0);
  switch (switches & 0x0000000C) { // Only consider switches SW3 and SW2
    case 0x00000004: j += 20480; break; // 10 MB
    case 0x00000008: j += 102400; break; // 50 MB
    case 0x0000000C: j += 204800; break; // 100 MB
    default: j += 10240; break; // 5 MB    
  }
   
  printf("Button 1 pressed. Forwarded (%d = %.2f MBs)...\n", j, (float)j / 2048.0);
}

void KEY2_Pressed() {
  char Text[16] = "Rewinded...     ";
  int switches;
    
  LCD_Line2();
  //  Show Text to LCD
  LCD_Show_Text(Text);
  
  // Rewind 5 MBs
  switches = IORD(SWITCH_PIO_BASE, 0);
  switch (switches & 0x0000000C) {
    case 0x00000004: // 10 MB
      if (j >= 20480) j -= 20480;    
    break; // 50 MB 
    case 0x00000008: 
      if (j >= 102400) j -= 102400;    
    break;
    case 0x0000000C:  // 100 MB
      if (j >= 204800) j -= 204800;    
    break;
    default:  // 5 MB    
      if (j >= 10242) j -= 10240;
    break;
  }

  printf("Button 2 pressed. Rewinded (%d = %.2f MBs)...\n", j, (float)j / 2048.0);
}

void KEY3_Pressed() {
  System_init();
  printf("Button 3 pressed. Status: %d\n", Status); 
}
//-------------------------------------------------------------------------
void handle_button_interrupts()
{
  outport(LED_GREEN_BASE,get_pio_edge_cap(BUTTON_PIO_BASE)*get_pio_edge_cap(BUTTON_PIO_BASE));
  switch(get_pio_edge_cap(BUTTON_PIO_BASE)) {
    case 1: KEY0_Pressed(); break;
    case 2: KEY1_Pressed(); break;
    case 4: KEY2_Pressed(); break;
    case 8: KEY3_Pressed(); break;
  }
  set_pio_edge_cap(BUTTON_PIO_BASE,0x0);
  usleep(BUTTON_STOP_TIME);
  outport(LED_GREEN_BASE,LED_GREEN_VALUE);
}
//-------------------------------------------------------------------------
void init_button_irq()
{
  /* Enable all 4 button interrupts. */
  set_pio_irq_mask(BUTTON_PIO_BASE, BUTTON_INT_MASK);
  /* Reset the edge capture register. */
  set_pio_edge_cap(BUTTON_PIO_BASE, 0x0);
  /* Register the interrupt handler. */
  alt_irq_register( BUTTON_PIO_IRQ, NULL, (void*)handle_button_interrupts ); 
}
//-------------------------------------------------------------------------
void basic_test()
{
    //  Show LCD Test Text
  LCD_Test();
  
  //  Refresh LED
  outport(LED_GREEN_BASE,LED_GREEN_VALUE);
  outport(LED_RED_BASE,LED_RED_VALUE);
  //  Refresh SEG7
  seg7_show(SEG7_DISPLAY_BASE,SEG7_VALUE);
  
  //  I2C Core Initial & Set Freq = 100 Khz
//  I2C_Init(10000000/I2C_FREQ-1);
  
  //  Check Video Encoder on I2C Bus, Address = 0x40
//  if(I2C_Send(0x40,1,1))
//  printf("\nFind Video Encoder on I2C Bus, Address = 0x40.\n");
//  else
//  printf("\nCan't Find Video Encoder on I2C Bus.\n");
  
  //  Check Audio CODEC on I2C Bus, Address = 0x34
//  if(I2C_Send(0x34,1,1))
//  printf("\nFind Audio CODEC on I2C Bus, Address = 0x34.\n");
//  else
//  printf("\nCan't Find Audio CODEC on I2C Bus.\n");

  //  Set Button PIO Interrupts
  init_button_irq();
}
//-------------------------------------------------------------------------
