// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================


`timescale 1 ns/ 100 ps

`include "defines.v"

`define EOF 32'hFFFF_FFFF
`define NULL 0
`define MAX_LINE_LENGTH 1000

`define NUM_MAC_SAMPLE 'd576
`define NUM_PCM_SAMPLE 'd576
`define NUM_OF_FRAME 'd412
`define NUM_OF_GRANULE 'd2
`define NUM_OF_CHANNEL 'd2

module mp3_top_tb_mp3_v_tf_tim();

// DATE:     10:46:06 07/25/2005 
// MODULE:   mp3_top
// DESIGN:   mp3_top
// FILENAME: tb_mp3.v
// PROJECT:  mp3_top
// VERSION:  


// Inputs
    reg MASTER_CLOCK_I;
    reg AC97_BIT_CLOCK_I;
    reg AC97_DATA_IN_I;
    reg USER_INPUT0_I;
    reg USER_INPUT1_I;
    reg PAL_NTSC;
    reg S_VIDEO;


// Outputs
    wire AC97_SYNCH_O;
    wire AC97_DATA_OUT_O;
    wire AC97_BEEP_TONE_O;
    wire USER_LED0_O;
    wire USER_LED1_O;
    wire STARTUP_O;

	 // for testing
	 wire MEM_resetn;
	 wire MEM_start;

	reg [`ADDRESS_WIDTH-1:0] LEFT_MAC_RAM_write_address;
	reg [`DATA_WIDTH-1:0] LEFT_MAC_RAM_write_data;
	reg LEFT_MAC_RAM_we;
	reg LEFT_MAC_RAM_en;
	reg [1:0] LEFT_MAC_block_type;

	reg [`ADDRESS_WIDTH-1:0] RIGHT_MAC_RAM_write_address;
	reg [`DATA_WIDTH-1:0] RIGHT_MAC_RAM_write_data;
	reg RIGHT_MAC_RAM_we;
	reg RIGHT_MAC_RAM_en;
	reg [1:0] RIGHT_MAC_block_type;

	reg MEM_done;
// Bidirs


// Instantiate the UUT
    mp3_top uut (
        .MASTER_CLOCK_I(MASTER_CLOCK_I), 
        .AC97_BIT_CLOCK_I(AC97_BIT_CLOCK_I), 
        .AC97_SYNCH_O(AC97_SYNCH_O), 
        .AC97_DATA_IN_I(AC97_DATA_IN_I), 
        .AC97_DATA_OUT_O(AC97_DATA_OUT_O), 
        .AC97_BEEP_TONE_O(AC97_BEEP_TONE_O), 
        .USER_INPUT0_I(USER_INPUT0_I), 
        .USER_INPUT1_I(USER_INPUT1_I), 
        .PAL_NTSC(PAL_NTSC), 
        .S_VIDEO(S_VIDEO), 
        .USER_LED0_O(USER_LED0_O), 
        .USER_LED1_O(USER_LED1_O), 
        .STARTUP_O(STARTUP_O),
		  // for testing without memory controller
			.MEM_resetn(MEM_resetn),
			.MEM_start(MEM_start),
			.LEFT_MAC_RAM_write_address(LEFT_MAC_RAM_write_address),
			.LEFT_MAC_RAM_write_data(LEFT_MAC_RAM_write_data),
			.LEFT_MAC_RAM_we(LEFT_MAC_RAM_we),
			.LEFT_MAC_RAM_en(LEFT_MAC_RAM_en),
			.LEFT_MAC_block_type(LEFT_MAC_block_type),
			
			.RIGHT_MAC_RAM_write_address(RIGHT_MAC_RAM_write_address),
			.RIGHT_MAC_RAM_write_data(RIGHT_MAC_RAM_write_data),
			.RIGHT_MAC_RAM_we(RIGHT_MAC_RAM_we),
			.RIGHT_MAC_RAM_en(RIGHT_MAC_RAM_en),
			.RIGHT_MAC_block_type(RIGHT_MAC_block_type),

			.MEM_done(MEM_done)
        );


// the clock signals
parameter master_cycle = 18.5;

always begin
	#(master_cycle);
	MASTER_CLOCK_I = 1'b0;
	#(master_cycle);
	MASTER_CLOCK_I = 1'b1;
end

always begin
	#(master_cycle*2.197265625);
	AC97_BIT_CLOCK_I = 1'b0;
	#(master_cycle*2.197265625);
	AC97_BIT_CLOCK_I = 1'b1;
end

task master_reset;
// task for generating master reset for two clock cycles
begin
	USER_INPUT0_I = 1'b1;

	wait (MASTER_CLOCK_I !== 1'bx);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT0_I = 1'b0;
	@ (posedge MASTER_CLOCK_I);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT0_I = 1'b1;
end
endtask

task other_reset;
// task for generating other reset for two clock cycles
begin
	USER_INPUT1_I = 1'b1;

	wait (MASTER_CLOCK_I !== 1'bx);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT1_I = 1'b0;
	@ (posedge MASTER_CLOCK_I);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT1_I = 1'b1;
end
endtask

// Initialize Inputs
integer i;

// file pointers
integer control_fp;
integer pcm_in_fp, pcm_out_fp;
integer mac_in_fp, mac_out_fp;
integer alias_out_fp;
integer mdct_out_fp, mdct_in_fp;
//integer wr_frame, wr_granule, wr_channel, wr_sample;

// global variables
integer frame, granule, channel, sample;

// for reading data from files										
reg [`DATA_WIDTH-1:0] pcm_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][`NUM_PCM_SAMPLE-1:0];
reg [`DATA_WIDTH-1:0] alias_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][`NUM_PCM_SAMPLE-1:0];
reg [`DATA_WIDTH-1:0] alias_block_type [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];

integer unmatch_count [`NUM_OF_CHANNEL-1:0];

task read_pcm_data;
// task for reading data from file for verification
integer frame, granule, channel, sample;
integer temp_frame, temp_granule, temp_channel, temp_sample;
integer pcm_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;
	temp_frame = 0;
	temp_granule = 0;
	temp_channel = 0;
	temp_sample = 0;


	pcm_data_fp = $fopen("TXT/debug_from_subsyn_16.txt", "r");

	if (pcm_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(pcm_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, pcm_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, pcm_data_fp);
         file_r = $fscanf(pcm_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

			pcm_data[frame][granule][channel][sample_num] = sample_hex;
			sample = sample + 1;
			if (sample_num == 575) begin
				$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
				sample = 0;
				if (channel == 0) begin
					channel = 1;
				end
				else begin
					if (granule == 0) begin
						granule = 1;
					end else begin
						granule = 0;
						frame = frame + 1;
					end
					channel = 0;
				end
			end
		end
		file_c = $fgetc(pcm_data_fp);
	end

	$fclose(pcm_data_fp);

	$display("Finish reading pcm data file...");
end
endtask

task read_alias_data;
// task for reading anti-alias data from file for verification
integer frame, granule, channel, sample;
integer temp_frame, temp_granule, temp_channel, temp_sample;
integer alias_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;
	temp_frame = 0;
	temp_granule = 0;
	temp_channel = 0;
	temp_sample = 0;


	alias_data_fp = $fopen("TXT/debug_alias_input_16.txt", "r");

	if (alias_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(alias_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, alias_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, alias_data_fp);
         file_r = $fscanf(alias_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

			if (sample_num == -1) begin
				alias_block_type[frame][granule][channel] = sample_hex;
//				$write("fr=%d gr=%d ch=%d block_type=%d\n", frame, granule, channel, alias_block_type[frame][granule][channel]);
			end else begin
				alias_data[frame][granule][channel][sample_num] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d sample_num=%d data=%h\n", frame, granule, channel, sample, sample_num, alias_data[frame][granule][channel][sample_num]);
				sample = sample + 1;
				if (sample_num == 575) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
			end
		end
		file_c = $fgetc(alias_data_fp);
	end

	$fclose(alias_data_fp);

	$display("Finish reading alias data file...");
end
endtask

integer log_fp;

task check_data;
input frame, granule, channel, sample;
integer frame, granule, channel, sample;
input got_data;
reg [`DATA_WIDTH-1:0] got_data;
begin
	if (got_data != pcm_data[frame][granule][channel][sample]) begin
		log_fp = $fopen("TXT/data_log.txt", "a");
		$fwrite(log_fp, "Data unmatched at fr=%d gr=%d ch=%d sample=%d got=%h expected=%h\n", frame, granule, channel, sample, got_data, pcm_data[frame][granule][channel][sample]);
		$write("Data unmatched at fr=%d gr=%d ch=%d sample=%d got=%h expected=%h\n", frame, granule, channel, sample, got_data, pcm_data[frame][granule][channel][sample]);
		$fclose(log_fp);
		unmatch_count[channel] = unmatch_count[channel] + 1;
	end
end
endtask

        initial begin
				// read expected data
				read_alias_data;
				read_pcm_data;

				frame = -2;
				channel = 0;
				granule = 0;
				sample = 0;
  				i = 0;
				unmatch_count[0] = 0;
				unmatch_count[1] = 0;

            AC97_DATA_IN_I = 1;
            USER_INPUT0_I = 1;
            USER_INPUT1_I = 1;
            PAL_NTSC = 1;
            S_VIDEO = 1;

				$display("INFO: Simulation started...");

				// Generate reset signals
				master_reset;
				other_reset;

				$display("INFO: Reset signals generated...");

				control_fp = $fopen("TXT/control.txt", "w");
				pcm_out_fp = $fopen("TXT/pcm_out_temp.txt", "w");
				pcm_in_fp = $fopen("TXT/pcm_in_temp.txt", "w");
				mac_out_fp = $fopen("TXT/mac_out_temp.txt", "w");
				mac_in_fp = $fopen("TXT/mac_in_temp.txt", "w");
				alias_out_fp = $fopen("TXT/alias_out_temp.txt", "w");
				mdct_in_fp = $fopen("TXT/mdct_in_temp.txt", "w");
				mdct_out_fp = $fopen("TXT/mdct_out_temp.txt", "w");
				log_fp = $fopen("TXT/data_log.txt", "w");
				 
				$fclose(control_fp);
				$fclose(pcm_out_fp);
				$fclose(pcm_in_fp);
				$fclose(mac_out_fp);
				$fclose(mac_in_fp);
				$fclose(alias_out_fp);
				$fclose(mdct_in_fp);
				$fclose(mdct_out_fp);
				$fclose(log_fp);

				$display("INFO: Program started...");
	     end

			integer alias_state;
			integer alias_sb_count;
			integer alias_counter;
			integer left_alias_frame;
			integer left_alias_granule;
			integer left_alias_sample;
			integer right_alias_frame;
			integer right_alias_granule;
			integer right_alias_sample;

		  always @ (negedge MEM_done) begin
			$write("MEM started...\n");
			$write("\t\t\t\t\t\t\t\t\t\tLEFT_MAC_RAM_read_en=%d\n", LEFT_MAC_RAM_en);
			$write("\t\t\t\t\t\t\t\t\t\tRIGHT_MAC_RAM_read_en=%d\n", RIGHT_MAC_RAM_en);
			$write("\t\t\t\t\t\t\t\t\t\tMEM_done=%d\n", MEM_done);
			$write("\t\t\t\t\t\t\t\t\t\talias_sb_count=%d\n", alias_sb_count);
			$write("\t\t\t\t\t\t\t\t\t\talias_counter=%d\n", alias_counter);
			$write("\t\t\t\t\t\t\t\t\tLEFT:\n");
			$write("\t\t\t\t\t\t\t\t\t\talias_frame=%d\n", left_alias_frame);
			$write("\t\t\t\t\t\t\t\t\t\talias_granule=%d\n", left_alias_granule);
			$write("\t\t\t\t\t\t\t\t\t\talias_sample=%d\n", left_alias_sample);
			$write("\t\t\t\t\t\t\t\t\tRIGHT:\n");
			$write("\t\t\t\t\t\t\t\t\t\talias_frame=%d\n", right_alias_frame);
			$write("\t\t\t\t\t\t\t\t\t\talias_granule=%d\n", right_alias_granule);
			$write("\t\t\t\t\t\t\t\t\t\talias_sample=%d\n", right_alias_sample);
		  end

		  always @ (posedge uut.MEM_done) begin
		  	$write("MEM finished...\n");
			$write("\t\t\t\t\t\t\t\t\t\tLEFT_block_type=%d\n", LEFT_MAC_block_type);
			$write("\t\t\t\t\t\t\t\t\t\tRIGHT_block_type=%d\n", RIGHT_MAC_block_type);
			$write("\t\t\t\t\t\t\t\t\t\talias_sb_count=%d\n", alias_sb_count);
			$write("\t\t\t\t\t\t\t\t\t\talias_counter=%d\n", alias_counter);
			$write("\t\t\t\t\t\t\t\t\tLEFT:\n");
			$write("\t\t\t\t\t\t\t\t\t\talias_frame=%d\n", left_alias_frame);
			$write("\t\t\t\t\t\t\t\t\t\talias_granule=%d\n", left_alias_granule);
			$write("\t\t\t\t\t\t\t\t\t\talias_sample=%d\n", left_alias_sample);
			$write("\t\t\t\t\t\t\t\t\tRIGHT:\n");
			$write("\t\t\t\t\t\t\t\t\t\talias_frame=%d\n", right_alias_frame);
			$write("\t\t\t\t\t\t\t\t\t\talias_granule=%d\n", right_alias_granule);
			$write("\t\t\t\t\t\t\t\t\t\talias_sample=%d\n", right_alias_sample);
		  end

			// for writing data to the DP RAM (MAC RAM)
			always @ (posedge uut.MAC_clock or negedge MEM_resetn) begin
		  		if (MEM_resetn == 0) begin
					alias_state <= 'd0;
					alias_sb_count <= 'd0;
					alias_counter <= 'd0;
					left_alias_frame <= 'd0;
					left_alias_granule <= 'd0;
					left_alias_sample <= 'd0;
					right_alias_frame <= 'd0;
					right_alias_granule <= 'd0;
					right_alias_sample <= 'd0;
					MEM_done <= 'd1;
					LEFT_MAC_RAM_write_address <= 'd0;
					LEFT_MAC_RAM_write_data <= 'd0;
					LEFT_MAC_RAM_we <= 'd0;
					LEFT_MAC_RAM_en <= 'd0;
					LEFT_MAC_block_type <= 'd0;
				
					RIGHT_MAC_RAM_write_address <= 'd0;
					RIGHT_MAC_RAM_write_data <= 'd0;
					RIGHT_MAC_RAM_we <= 'd0;
					RIGHT_MAC_RAM_en <= 'd0;
					RIGHT_MAC_block_type <= 'd0;
				end else begin
					if (MEM_start == 1) begin
						alias_state <= 'd1;
						LEFT_MAC_block_type <= alias_block_type[left_alias_frame][left_alias_granule][0];
						RIGHT_MAC_block_type <= alias_block_type[right_alias_frame][right_alias_granule][1];
						alias_sb_count <= 'd0;
						alias_counter <= 'd0;
						left_alias_sample <= 'd0;
						right_alias_sample <= 'd0;
					end else begin
						if ((alias_sb_count * 32 + alias_counter) == 'd1023) begin
							alias_state <= 'd0;
							alias_sb_count <= 'd0;
							alias_counter <= 'd0;
							left_alias_sample <= 'd0;
							right_alias_sample <= 'd0;
						end
					end

					if (alias_state == 'd0) MEM_done <= 'd1;
					else MEM_done <= 'd0;

					if (alias_state == 'd0) begin
						LEFT_MAC_RAM_we <= 'd0;
						LEFT_MAC_RAM_en <= 'd0;

						RIGHT_MAC_RAM_we <= 'd0;
						RIGHT_MAC_RAM_en <= 'd0;
					end else begin
						LEFT_MAC_RAM_we <= 'd1;
						LEFT_MAC_RAM_en <= 'd1;

						RIGHT_MAC_RAM_we <= 'd1;
						RIGHT_MAC_RAM_en <= 'd1;
					end

					if (alias_state == 'd0) begin
						alias_counter <= 'd0;
						alias_sb_count <= 'd0;
					end else begin
						if (alias_counter < 'd31) begin
							alias_counter <= alias_counter + 1;
						end else begin
							alias_sb_count <= alias_sb_count + 1;
							alias_counter <= 'd0;
						end
					end

					if (alias_state == 'd1) begin
						if (LEFT_MAC_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								if (left_alias_sample == `NUM_MAC_SAMPLE - 1) begin
									if (left_alias_granule == 'd0) left_alias_granule <= 'd1;
									else begin
										left_alias_frame <= left_alias_frame + 1;
										left_alias_granule <= 'd0;
										left_alias_sample <= 'd0;
									end
								end else begin
									left_alias_sample <= left_alias_sample + 1;
								end
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									if (left_alias_sample == `NUM_MAC_SAMPLE-1) begin
										if (left_alias_granule == 'd0) left_alias_granule <= 'd1;
										else begin
											left_alias_frame <= left_alias_frame + 1;
											left_alias_granule <= 'd0;
											left_alias_sample <= 'd0;
										end
									end else begin
										left_alias_sample <= left_alias_sample + 1;
									end
								end
							end
						end
					end

					if (alias_state == 'd1) begin
						if (RIGHT_MAC_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								if (right_alias_sample == `NUM_MAC_SAMPLE - 1) begin
									if (right_alias_granule == 'd0) right_alias_granule <= 'd1;
									else begin
										right_alias_frame <= right_alias_frame + 1;
										right_alias_granule <= 'd0;
										right_alias_sample <= 'd0;
									end
								end else begin
									right_alias_sample <= right_alias_sample + 1;
								end
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									if (right_alias_sample == `NUM_MAC_SAMPLE-1) begin
										if (right_alias_granule == 'd0) right_alias_granule <= 'd1;
										else begin
											right_alias_frame <= right_alias_frame + 1;
											right_alias_granule <= 'd0;
											right_alias_sample <= 'd0;
										end
									end else begin
										right_alias_sample <= right_alias_sample + 1;
									end
								end
							end
						end
					end

					if (alias_state == 'd0) begin
						LEFT_MAC_RAM_write_address <= 'd0;

						RIGHT_MAC_RAM_write_address <= 'd0;
					end else begin
						LEFT_MAC_RAM_write_address <= alias_counter + alias_sb_count * 32;

						RIGHT_MAC_RAM_write_address <= alias_counter + alias_sb_count * 32;
					end
					
					if (alias_state == 'd1) begin
						if (LEFT_MAC_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								LEFT_MAC_RAM_write_data <= alias_data[left_alias_frame][left_alias_granule][0][(left_alias_sample + 9) % `NUM_MAC_SAMPLE];
							end else begin
								LEFT_MAC_RAM_write_data <= LEFT_MAC_block_type;
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									LEFT_MAC_RAM_write_data <= alias_data[left_alias_frame][left_alias_granule][0][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3];
								end else begin
									LEFT_MAC_RAM_write_data <= LEFT_MAC_block_type;
								end
							end else begin
								LEFT_MAC_RAM_write_data <= LEFT_MAC_block_type;
							end
						end
					end

					if (alias_state == 'd1) begin
						if (RIGHT_MAC_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								RIGHT_MAC_RAM_write_data <= alias_data[right_alias_frame][right_alias_granule][1][(right_alias_sample + 9) % `NUM_MAC_SAMPLE];
							end else begin
								RIGHT_MAC_RAM_write_data <= RIGHT_MAC_block_type;
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									RIGHT_MAC_RAM_write_data <= alias_data[right_alias_frame][right_alias_granule][1][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3];
								end else begin
									RIGHT_MAC_RAM_write_data <= RIGHT_MAC_block_type;
								end
							end else begin
								RIGHT_MAC_RAM_write_data <= RIGHT_MAC_block_type;
							end
						end
					end
/*
					if (alias_state == 'd1) begin
						$write("l: sb_count=%d count=%d fr=%d gr=%d sample=%d short_sample=%d en=%d we=%d addr=%d di=%h\n",
								alias_sb_count,
								alias_counter,
								left_alias_frame,
								left_alias_granule,
								left_alias_sample,
								(alias_sb_count*18)+(alias_counter/8)+(alias_counter%8)*3,
								LEFT_MAC_RAM_en,
								LEFT_MAC_RAM_we,
								LEFT_MAC_RAM_write_address,
								LEFT_MAC_RAM_write_data
								);
					end
*/
				end
			end

			// for timing
					  always @ (posedge AC97_SYNCH_O) begin
		  		// Print read PCM sample
		  		if (i > 15) begin
					pcm_out_fp = $fopen("TXT/pcm_out_temp.txt", "a");
					$fwrite(pcm_out_fp, "READ_PCM: (%d, %d) ra=%d, ld=%h (%d), rd=%h (%d)\n", frame, granule, uut.ac97_module_sample_counter, uut.LEFT_PCM_RAM_read_data, uut.LEFT_PCM_RAM_read_data, uut.RIGHT_PCM_RAM_read_data, uut.RIGHT_PCM_RAM_read_data);
					$fclose(pcm_out_fp);

					if (sample % 100 == 0) $write("Sample read: %d\n", sample);

					if (frame >= 0) begin
						check_data(frame, granule, 0, sample, uut.LEFT_PCM_RAM_read_data);
						check_data(frame, granule, 1, sample, uut.RIGHT_PCM_RAM_read_data);
					end

					sample = sample+1;

					if (sample == `NUM_PCM_SAMPLE) begin
						$write("AC97 finished reading (fr=%d gr=%d) from PCM_RAM...\n", frame, granule);
						if (frame >= 0) begin
							$write("Last left value in frame: got=%h expected=%h\n", uut.LEFT_PCM_RAM_read_data, pcm_data[frame][granule][0][sample-1]);
							$write("Last right value in frame: got=%h expected=%h\n", uut.RIGHT_PCM_RAM_read_data, pcm_data[frame][granule][1][sample-1]);
							$write("Channel 0: Unmatch count=%d\n", unmatch_count[0]);
							$write("Channel 1: Unmatch count=%d\n", unmatch_count[1]);
							if (granule == 0) granule = 1;
							else begin
								granule = 0;
								frame = frame + 1;
							end
						end else
						frame = frame + 1;
						sample = 0;
					end

					if (frame == `NUM_OF_FRAME) begin
						# 1000;
						log_fp = $fopen("TXT/data_log.txt", "a");
						$fwrite(log_fp, "\n\nTotal unmatch count: ch0=%d ch1=%d\n", unmatch_count[0], unmatch_count[1]);
						$write("\n\nTotal unmatch count: ch0=%d ch1=%d\n", unmatch_count[0], unmatch_count[1]);
						$fclose(log_fp);
						$stop(2);
					end
				end
		  end

		  always @ (posedge AC97_SYNCH_O) begin
		  		if (i > 15) #10;
				else 
				i = i+1;
		
//					$write("INFO: %d: %d\n", i, uut.MAC_module_MAC_RAM_ADDRESS_O);
				#10;
		  end
/*
			always @ (posedge uut.MAC_clock) begin
			if (uut.LEFT_MAC_start == 1'b1 && uut.MAC_done == 1'b0) begin
				$write("LEFT MAC started...\n");
				$write("\t\t\t\t\t\t\t\t\t\tstate=%d\n", uut.MAC_module.left_state);
				$write("\t\t\t\t\t\t\t\t\t\tblock_type=%d\n", uut.MAC_module.LEFT_MAC_BLOCK_I);
				$write("\t\t\t\t\t\t\t\t\t\thold=%d\n", uut.MAC_module.left_hold);
				$write("\t\t\t\t\t\t\t\t\t\tcounter=%d\n", uut.MAC_module.left_counter);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_pointer=%d\n", uut.MAC_module.left_aic_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_offset=%d\n", uut.MAC_module.left_aic_read_offset);
				$write("\t\t\t\t\t\t\t\t\t\ticg_read_pointer=%d\n", uut.MAC_module.left_icg_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_write_pointer=%d\n", uut.MAC_module.left_antialias_write_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_read_pointer=%d\n", uut.MAC_module.left_antialias_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tbackup_pointer=%d\n", uut.MAC_module.left_backup_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tMAC_RAM_pointer=%d\n", uut.MAC_module.LEFT_MAC_RAM_pointer);
				$write("\n");			
			end
			if (uut.RIGHT_MAC_start == 1'b1 && uut.MAC_done == 1'b0) begin
				$write("RIGHT MAC started...\n");
				$write("\t\t\t\t\t\t\t\t\t\tstate=%d\n", uut.MAC_module.right_state);
				$write("\t\t\t\t\t\t\t\t\t\tblock_type=%d\n", uut.MAC_module.RIGHT_MAC_BLOCK_I);
				$write("\t\t\t\t\t\t\t\t\t\thold=%d\n", uut.MAC_module.right_hold);
				$write("\t\t\t\t\t\t\t\t\t\tcounter=%d\n", uut.MAC_module.right_counter);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_pointer=%d\n", uut.MAC_module.right_aic_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_offset=%d\n", uut.MAC_module.right_aic_read_offset);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_write_pointer=%d\n", uut.MAC_module.right_antialias_write_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_read_pointer=%d\n", uut.MAC_module.right_antialias_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tbackup_pointer=%d\n", uut.MAC_module.right_backup_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tMAC_RAM_pointer=%d\n", uut.MAC_module.RIGHT_MAC_RAM_pointer);
			end
			if (uut.LEFT_MAC_start == 1'b1) begin
				$write("\n");			
				$write("\t\t\t\t\t\t\t\t\t\tPCM_RAM_pointer=%d\n", uut.MAC_module.PCM_RAM_pointer);
			end
		  end


		  always @ (posedge uut.MAC_done) begin
		  	$write("MAC finished...wab=%d\n", uut.LEFT_PCM_RAM_write_address_buffer);
		  end
*/
/*
		  always @ (posedge uut.MAC_clock) begin
		  		// Control signals
				control_fp = $fopen("TXT/control.txt", "a");
				$fwrite(control_fp, "MEM_start=%d, MEM_done=%d, MAC_RAM_read_en=%d, MAC_RAM_write_address=%d, MAC_block_type=%d, ",
							uut.MEM_start, 
							uut.MEM_done, 
							uut.MAC_RAM_read_en,
							uut.MAC_RAM_write_address,
							uut.MAC_block_type
							);
				$fwrite(control_fp, "MAC_start=%d, MAC_done=%d, PCM_RAM_read_address_buffer=%d, PCM_RAM_write_address_buffer=%d\n", 
							uut.MAC_start, 
							uut.MAC_done,
							uut.PCM_RAM_read_address_buffer,
							uut.PCM_RAM_write_address_buffer
							);

				$fclose(control_fp);
		  end
*/
/*
	reg [9:0] address_buf;

		always @ (posedge uut.MAC_clock) begin
			address_buf <= uut.MAC_module.LEFT_MAC_RAM_ADDRESS_O;
		end

		always @(posedge uut.MAC_clock) begin
		  	// input of antialias
			if ((uut.MAC_module.left_state == `STATE_LOAD_HALF_SB) 
			|| (uut.MAC_module.left_state == `STATE_ANTIALIAS)
			|| (uut.MAC_module.left_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
			|| (uut.MAC_module.left_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
			|| (uut.MAC_module.left_state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
			) 
			begin
	   	$display("ANTI_IN: left_state=%d counter=%2d en=%1d ra=%4d do=%4h ed=%h", 
				uut.MAC_module.left_state, 
				uut.MAC_module.left_counter, 
				uut.MAC_module.LEFT_MAC_RAM_EN_O,
				uut.MAC_module.LEFT_MAC_RAM_ADDRESS_O,
				uut.MAC_module.LEFT_MAC_RAM_DATA_I,
				left_ram_data[address_buf]);
			if (left_ram_data[address_buf] != uut.MAC_module.LEFT_MAC_RAM_DATA_I) begin
				$display("Data unmatched: addr=%d got=%h expected=%h", address_buf, uut.MAC_module.LEFT_MAC_RAM_DATA_I, left_ram_data[(uut.MAC_module.LEFT_MAC_RAM_ADDRESS_O-1)%1024]);
			end

			end
		end
*/
/*
		  always @(posedge uut.MAC_clock) begin
		  	// output of antialias
			if ((uut.MAC_module.left_state == `STATE_LOAD_HALF_SB || uut.MAC_module.left_state == `STATE_ANTIALIAS) 
			&& uut.MAC_module.left_icg_wr_en == 1'b1)
			begin

	   	$display("ANTI_OUT: left_state=%d counter=%2d ra1=%4d ra2=%4d di=%4h da=%4h db=%4h mac=%1d wp=%4d we=%1b wa=%4d di=%4h", 
				uut.MAC_module.left_state, 
				uut.MAC_module.left_counter, 
				uut.MAC_module.left_aic_address, 
				uut.MAC_module.LEFT_MAC_RAM_ADDRESS_O,
				uut.MAC_module.LEFT_MAC_RAM_DATA_I,
				uut.MAC_module.left_mult_accum_data_a,
				uut.MAC_module.left_mult_accum_data_b,
				uut.MAC_module.left_add_accum_control,
				uut.MAC_module.left_antialias_write_pointer,
				uut.MAC_module.left_icg_wr_en, 
				uut.MAC_module.left_icg_address,
				uut.MAC_module.left_acc_out);

			end
		end
*/
/*
	 always @(posedge uut.MAC_clock) begin
		  	// output of antialias
			if ((uut.MAC_module.right_state == `STATE_LOAD_HALF_SB || uut.MAC_module.right_state == `STATE_ANTIALIAS) 
			&& uut.MAC_module.right_icg_wr_en == 1'b1)
			begin

	   	$display("ANTI_OUT: right_state=%d counter=%2d ra1=%4d ra2=%4d da=%4h db=%4h mac=%1d wp=%4d we=%1b wa=%4d di=%4h", 
				uut.MAC_module.right_state, 
				uut.MAC_module.right_counter, 
				uut.MAC_module.right_aic_address, 
				uut.MAC_module.RIGHT_MAC_RAM_ADDRESS_O,
				uut.MAC_module.right_mult_accum_data_a,
				uut.MAC_module.right_mult_accum_data_b,
				uut.MAC_module.right_add_accum_control,
				uut.MAC_module.right_antialias_write_pointer,
				uut.MAC_module.right_icg_wr_en, 
				uut.MAC_module.right_icg_address,
				uut.MAC_module.right_acc_out);

			end
		end
*/
/*
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for normal block
		if ((uut.MAC_module.left_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE)
		 && (uut.MAC_module.left_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.left_add_accum_control,
				uut.MAC_module.left_bypass_mult,
				uut.MAC_module.left_control_mux_a, 
				uut.MAC_module.left_control_mux_b, 
				uut.MAC_module.left_mult_accum_data_a,
				uut.MAC_module.left_mult_accum_data_b,
				uut.MAC_module.left_icg_read_pointer,				
				uut.MAC_module.left_icg_mem_en, 
				uut.MAC_module.left_icg_wr_en, 
				uut.MAC_module.left_icg_address,
				uut.MAC_module.left_icg_data_in,
				uut.MAC_module.left_icg_data_out);
	end
*/
/*
		always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for short block
		if (((uut.MAC_module.left_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.left_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.left_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)))
//		 && (uut.MAC_module.left_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t addr=%d \t\t en=%d \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.left_add_accum_control,
				uut.MAC_module.left_bypass_mult,
				uut.MAC_module.left_control_mux_a, 
				uut.MAC_module.left_control_mux_b, 
				uut.MAC_module.left_mult_accum_data_a,
				uut.MAC_module.LEFT_MAC_RAM_ADDRESS_O,
				uut.MAC_module.LEFT_MAC_RAM_EN_O,
				uut.MAC_module.left_mult_accum_data_b,
				uut.MAC_module.left_icg_read_pointer,				
				uut.MAC_module.left_icg_mem_en, 
				uut.MAC_module.left_icg_wr_en, 
				uut.MAC_module.left_icg_address,
				uut.MAC_module.left_icg_data_in,
				uut.MAC_module.left_icg_data_out);
	end
*/

/*
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
//		if (uut.MAC_module.left_state == `STATE_IMDCT_FREQ_INV)
		if (uut.MAC_module.left_icg_wr_en == 1'b1)
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.left_add_accum_control,
				uut.MAC_module.left_bypass_mult,
				uut.MAC_module.left_control_mux_a, 
				uut.MAC_module.left_control_mux_b, 
				uut.MAC_module.left_mult_accum_data_a,
				uut.MAC_module.left_mult_accum_data_b,
				uut.MAC_module.left_icg_read_pointer,				
				uut.MAC_module.left_icg_mem_en, 
				uut.MAC_module.left_icg_wr_en, 
				uut.MAC_module.left_icg_address,
				uut.MAC_module.left_icg_data_in,
				uut.MAC_module.left_icg_data_out);
	end
*/
/*
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
//		if (uut.MAC_module.left_state == `STATE_IMDCT_FREQ_INV)
//		if (uut.MAC_module.right_icg_wr_en == 1'b1)
		if (uut.MAC_module.right_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE)
	begin
		mdct_in_fp = $fopen("TXT/mdct_in_temp.txt", "a");
		$fwrite(mdct_in_fp, "RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.right_counter,				
				uut.MAC_module.right_state, 
				uut.MAC_module.right_add_accum_control,
				uut.MAC_module.right_bypass_mult,
				uut.MAC_module.right_control_mux_a, 
				uut.MAC_module.right_control_mux_b, 
				uut.MAC_module.right_mult_accum_data_a,
				uut.MAC_module.right_mult_accum_data_b,
				uut.MAC_module.right_icg_mem_en, 
				uut.MAC_module.right_icg_wr_en, 
				uut.MAC_module.right_icg_address,
				uut.MAC_module.right_icg_data_in,
				uut.MAC_module.right_icg_data_out);
		$fclose(mdct_in_fp);
				 
	   	$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t mo=%h \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.right_counter,				
				uut.MAC_module.right_state, 
				uut.MAC_module.right_add_accum_control,
				uut.MAC_module.right_bypass_mult,
				uut.MAC_module.right_control_mux_a, 
				uut.MAC_module.right_control_mux_b, 
				uut.MAC_module.right_mult_accum_data_a,
				uut.MAC_module.right_mult_accum_data_b,
				uut.MAC_module.instance_right_mult_accum.mult16_module.mult_out,
				uut.MAC_module.right_icg_mem_en, 
				uut.MAC_module.right_icg_wr_en, 
				uut.MAC_module.right_icg_address,
				uut.MAC_module.right_icg_data_in,
				uut.MAC_module.right_icg_data_out);

	end
*/
/*
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// input to MDCT
	if ((uut.MAC_module.left_state == `STATE_POLYPHASE_MDCT) && (uut.MAC_module.left_counter[9:5] == 0))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t do=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.left_add_accum_control,
				uut.MAC_module.left_bypass_mult,
				uut.MAC_module.left_control_mux_a, 
				uut.MAC_module.left_control_mux_b, 
				uut.MAC_module.left_mult_accum_data_a,
				uut.MAC_module.left_mult_accum_data_b,
				uut.MAC_module.left_icg_read_pointer,				
				uut.MAC_module.left_icg_mem_en, 
				uut.MAC_module.left_icg_wr_en, 
				uut.MAC_module.left_icg_address,
				uut.MAC_module.left_icg_data_out);
	end
*/
/*
always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// input to MDCT
	if ((uut.MAC_module.left_state == `STATE_POLYPHASE_MDCT) && (uut.MAC_module.left_counter[9:5] == 0) && (uut.MAC_module.right_state == `STATE_WAIT_OTHER_CHANNEL))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t do=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.right_add_accum_control,
				uut.MAC_module.right_bypass_mult,
				uut.MAC_module.right_control_mux_a, 
				uut.MAC_module.right_control_mux_b, 
				uut.MAC_module.right_mult_accum_data_a,
				uut.MAC_module.right_mult_accum_data_b,
				uut.MAC_module.right_icg_read_pointer,				
				uut.MAC_module.right_icg_mem_en, 
				uut.MAC_module.right_icg_wr_en, 
				uut.MAC_module.right_icg_address,
				uut.MAC_module.right_icg_data_out);
	end
*/
/*
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of polyphase windowing
	if (uut.MAC_module.left_state == `STATE_POLYPHASE_WINDOW 
	&& uut.MAC_module.LEFT_PCM_RAM_WE_O)
	begin
	 	$display("LEFT : counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t ca=%d \t\t cd=%h \t\t ra=%d \t\t di=%h \t\t we=%d \t\t wa=%d \t\t wd=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.left_add_accum_control,
				uut.MAC_module.left_bypass_mult,
				uut.MAC_module.left_control_mux_a, 
				uut.MAC_module.left_control_mux_b, 
				uut.MAC_module.left_mult_accum_data_a,
				uut.MAC_module.left_mult_accum_data_b,
				uut.MAC_module.pwc_address,
				uut.MAC_module.pwc_data_out,
				uut.MAC_module.left_psb_address,
				uut.MAC_module.left_psb_data_out,
				uut.MAC_module.LEFT_PCM_RAM_WE_O,
				uut.MAC_module.LEFT_PCM_RAM_ADDRESS_O,
				uut.MAC_module.LEFT_PCM_RAM_DATA_O
				);
	end
*/
/*
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of polyphase windowing
	if (uut.MAC_module.left_state == `STATE_POLYPHASE_WINDOW)
	begin
		$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t ca=%d \t\t cd=%h \t\t ra=%d \t\t di=%h \t\t we=%d \t\t wa=%d \t\t wd=%h", 
				uut.MAC_module.left_counter,				
				uut.MAC_module.left_state, 
				uut.MAC_module.right_add_accum_control,
				uut.MAC_module.right_bypass_mult,
				uut.MAC_module.right_control_mux_a, 
				uut.MAC_module.right_control_mux_b, 
				uut.MAC_module.right_mult_accum_data_a,
				uut.MAC_module.right_mult_accum_data_b,
				uut.MAC_module.pwc_address,
				uut.MAC_module.pwc_data_out,
				uut.MAC_module.right_psb_address,
				uut.MAC_module.right_psb_data_out,
				uut.MAC_module.RIGHT_PCM_RAM_WE_O,
				uut.MAC_module.RIGHT_PCM_RAM_ADDRESS_O,
				uut.MAC_module.RIGHT_PCM_RAM_DATA_O
				);
	end
*/
/*
	always @ (uut.MAC_module.state) begin
	if ((uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST) 
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_LOAD_IPG))
		$display("\t");
	end
*/
endmodule

