// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================


`timescale 1 ns/ 100 ps

`include "defines.v"

module mp3_top (
	MASTER_CLOCK_I,
//	ALTERNATE_CLOCK_I,
	AC97_BIT_CLOCK_I,
	AC97_SYNCH_O,
	AC97_DATA_IN_I,
	AC97_DATA_OUT_O,
	AC97_BEEP_TONE_O,
	USER_INPUT0_I,
	USER_INPUT1_I,
	PAL_NTSC,
	S_VIDEO,
	USER_LED0_O,			 
	USER_LED1_O,
	STARTUP_O,

   // ZBT RAM INTERFACE                           
   MEMORY_BANK0_CLK_P, 		MEMORY_BANK1_CLK_P,
   MEMORY_BANK0_CLKEN_N, 	MEMORY_BANK1_CLKEN_N,
   MEMORY_BANK0_WEN_N,		MEMORY_BANK1_WEN_N,
   MEMORY_BANK0_WENA_N,		MEMORY_BANK1_WENA_N,
   MEMORY_BANK0_WENB_N,		MEMORY_BANK1_WENB_N,
   MEMORY_BANK0_WENC_N,		MEMORY_BANK1_WENC_N,
   MEMORY_BANK0_WEND_N,		MEMORY_BANK1_WEND_N,
   MEMORY_BANK0_ADV_LD_N, 	MEMORY_BANK1_ADV_LD_N,
   MEMORY_BANK0_OEN_N, 		MEMORY_BANK1_OEN_N,
   MEMORY_BANK0_CEN_N, 		MEMORY_BANK1_CEN_N,
   MEMORY_BANK0_ADDR_P,		MEMORY_BANK1_ADDR_P,
   MEMORY_BANK0_DATA_A_P, 	MEMORY_BANK1_DATA_A_P, 
   MEMORY_BANK0_DATA_B_P, 	MEMORY_BANK1_DATA_B_P,
   MEMORY_BANK0_DATA_C_P, 	MEMORY_BANK1_DATA_C_P,
   MEMORY_BANK0_DATA_D_P, 	MEMORY_BANK1_DATA_D_P,

   // ETHERNET INTERFACE
   TX_DATA_P,
   TX_ENABLE_P,
   TX_CLOCK_P,
   TX_ERROR_P,
   ENET_SLEW_P,
   RX_DATA_P,
   RX_DATA_VALID_P,
   RX_ERROR_P,
   RX_CLOCK_P,
   COLLISION_DETECTED_P,
   CARRIER_SENSE_P,
   PAUSE_P,
   MDIO_P,
   MDC_P,
   MDINIT_N,
   SSN_DATA_P,
	MEM_CLK_FBIN_P,
	MEM_CLK_FBOUT_P
);

    input MASTER_CLOCK_I;
	 input MEM_CLK_FBIN_P;
	 output MEM_CLK_FBOUT_P;
//	 input ALTERNATE_CLOCK_I;
    input AC97_BIT_CLOCK_I;		// AC97 clock
    output AC97_SYNCH_O;			// AC97 synh output
    input AC97_DATA_IN_I;			// AC97 data in
    output AC97_DATA_OUT_O;		// AC97 data out
    output AC97_BEEP_TONE_O;		// AC97 Beep setting
    input USER_INPUT0_I;			// reset signal
    input USER_INPUT1_I;
	 input PAL_NTSC;
	 input S_VIDEO;
    output USER_LED0_O;
    output USER_LED1_O;
    output STARTUP_O;

// ZBT RAM INTERFACE
output 					MEMORY_BANK0_CLK_P,		MEMORY_BANK1_CLK_P;		// ZBT RAM clock
output 					MEMORY_BANK0_CLKEN_N,	MEMORY_BANK1_CLKEN_N;	// ZBT RAM clock enable
output 					MEMORY_BANK0_WEN_N,		MEMORY_BANK1_WEN_N;		// ZBT RAM write enable
output 					MEMORY_BANK0_WENA_N,		MEMORY_BANK1_WENA_N;		// ZBT RAM byte A write enable
output 					MEMORY_BANK0_WENB_N,		MEMORY_BANK1_WENB_N;		// ZBT RAM byte B write enable
output 					MEMORY_BANK0_WENC_N,		MEMORY_BANK1_WENC_N;		// ZBT RAM byte C write enable
output 					MEMORY_BANK0_WEND_N,		MEMORY_BANK1_WEND_N;		// ZBT RAM byte D write enable
output 					MEMORY_BANK0_ADV_LD_N,	MEMORY_BANK1_ADV_LD_N;	// ZBT RAM address load-advance
output 					MEMORY_BANK0_OEN_N,		MEMORY_BANK1_OEN_N;		// ZBT RAM output enable
output              	MEMORY_BANK0_CEN_N,		MEMORY_BANK1_CEN_N;		// ZBT RAM chip enable
output   [18:0]     	MEMORY_BANK0_ADDR_P,		MEMORY_BANK1_ADDR_P;		// ZBT RAM address bus
inout    [7:0] 		MEMORY_BANK0_DATA_A_P,	MEMORY_BANK1_DATA_A_P;	// ZBT RAM data bus byte "A"
inout    [7:0] 		MEMORY_BANK0_DATA_B_P,	MEMORY_BANK1_DATA_B_P;	// ZBT RAM data bus byte "B"
inout    [7:0] 		MEMORY_BANK0_DATA_C_P,	MEMORY_BANK1_DATA_C_P;	// ZBT RAM data bus byte "C"
inout    [7:0] 		MEMORY_BANK0_DATA_D_P,	MEMORY_BANK1_DATA_D_P;	// ZBT RAM data bus byte "D"

// ETHERNET INTERFACE
output   [3:0]       TX_DATA_P;              // Ethernet transmission data
output               TX_ENABLE_P;			// Ehternet transmission enable
input                TX_CLOCK_P;			// Ethernet transmission clock
input                TX_ERROR_P;			// Ethernet transmission error
output   [1:0]       ENET_SLEW_P;			// Ethernet slew settings
input    [3:0]       RX_DATA_P;			// Ethernet receive data
input                RX_DATA_VALID_P;		// Ethernet data valid
input                RX_ERROR_P;			// Ethernet receive error
input                RX_CLOCK_P;			// Ethernet receive clock
input                COLLISION_DETECTED_P;	// Ethernet collision detected
input                CARRIER_SENSE_P;		// Ethernet carrier sense
output               PAUSE_P;             	// Ethernet pause
inout                MDIO_P;              	// Ethernet config data
output               MDC_P;               	// Ethernet config clock
input                MDINIT_N;            	// Ethernet config init
inout                SSN_DATA_P;          	// Silicon serial number access

	// Clock trees
	reg 								Clock_resetn;
	wire 								Clock_ready;
	wire 								MAC_clock;
	wire 								HUFF_clock;
	wire 								AC97_bit_clock;
	wire 								master_clock;
	wire                       clk_27mhz;
	wire                       memory_27mhz;
   
	// For memory controller
	reg 								ETH_resetn;
	reg 								ETH_start;
	wire 								ETH_done;
	wire [`ADDRESS_WIDTH-1:0] 	ETH_DP_ADDRESS;
	wire [`DATA_WIDTH-1:0] 		ETH_DP_DATA;
	wire 								ETH_DP_WE;
	wire 								ETH_DP_EN;
	wire 								ETH_busy;
	wire 								ETH_range_end;

	reg								MP3_end_of_song;
	wire [1:0] 						MP3_Info_Mode;
	reg [1:0] 						MP3_Info_Mode_buffer;
	reg [1:0] 						MP3_Info_Mode_MAC_clock;
	wire [1:0] 						MP3_Info_Frequency;
	reg [1:0] 						MP3_Info_Frequency_buffer;

	// for Huff/Dequantizer
	reg 								HUFF_resetn;
	wire								HUFF_done;
	wire 								MAC_RAM_Idle;

	wire [`DATA_WIDTH-1:0] 		HUFF_DP_DATA;
	wire [`ADDRESS_WIDTH-1:0] 	HUFF_DP_ADDRESS;

	//for REQ RAMs
	wire 								Req_RAM_we;
	wire [`DATA_WIDTH-1:0] 		Req_RAM_write_data;
	wire [`DATA_WIDTH-1:0] 		Req_RAM_read_data;
	wire [`ADDRESS_WIDTH-1:0] 	Req_RAM_address;

	wire 								BR_RAM_we;
	wire [`DATA_WIDTH-1:0] 		BR_RAM_write_data;
	wire [`DATA_WIDTH-1:0] 		BR_RAM_read_data;
	wire [`ADDRESS_WIDTH-3:0] 	BR_RAM_address;
	
	wire [`ADDRESS_WIDTH-1:0] 	MAC_RAM_write_address;
	wire [`DATA_WIDTH-1:0] 		MAC_RAM_write_data;
	wire 								CH0_MAC_RAM_we;
	wire 								CH1_MAC_RAM_we;

	// for MAC
	wire [3:0]						CH0_WBBM;
	reg [1:0] 						MAC_state;
	wire [1:0] 						MP3_Info_CH0_MAC_block_type;
	wire 		 						MP3_Info_CH0_MAC_mixed_block;
	wire 		 						MP3_Info_CH0_MAC_switching;
	reg [1:0] 						MP3_Info_CH0_MAC_block_type_buffer;
	reg 		 						MP3_Info_CH0_MAC_mixed_block_buffer;
	reg 		 						MP3_Info_CH0_MAC_switching_buffer;
	reg [1:0] 						MP3_Info_CH0_MAC_block_type_MAC_clock;
	reg 		 						MP3_Info_CH0_MAC_mixed_block_MAC_clock;
	reg 		 						MP3_Info_CH0_MAC_switching_MAC_clock;

	assign MP3_Info_CH0_MAC_switching = CH0_WBBM[3];
	assign MP3_Info_CH0_MAC_block_type = CH0_WBBM[2:1];
	assign MP3_Info_CH0_MAC_mixed_block = CH0_WBBM[0];

	wire [3:0]						CH1_WBBM;
	wire [1:0] 						MP3_Info_CH1_MAC_block_type;
	wire 		 						MP3_Info_CH1_MAC_mixed_block;
	wire 		 						MP3_Info_CH1_MAC_switching;
	reg [1:0] 						MP3_Info_CH1_MAC_block_type_buffer;
	reg 		 						MP3_Info_CH1_MAC_mixed_block_buffer;
	reg 		 						MP3_Info_CH1_MAC_switching_buffer;
	reg [1:0] 						MP3_Info_CH1_MAC_block_type_MAC_clock;
	reg 		 						MP3_Info_CH1_MAC_mixed_block_MAC_clock;
	reg 		 						MP3_Info_CH1_MAC_switching_MAC_clock;

	assign MP3_Info_CH1_MAC_switching = CH1_WBBM[3];
	assign MP3_Info_CH1_MAC_block_type = CH1_WBBM[2:1];
	assign MP3_Info_CH1_MAC_mixed_block = CH1_WBBM[0];

	reg 								MAC_resetn;

	reg 								CH0_MAC_start;
	wire [`ADDRESS_WIDTH-1:0] 	CH0_MAC_RAM_read_address;
	wire [`DATA_WIDTH-1:0] 		CH0_MAC_RAM_read_data;
	wire 								CH0_MAC_RAM_read_en_MAC_clock;
	reg 								CH0_MAC_RAM_read_en_HUFF_clock;
	reg [`ADDRESS_WIDTH-1:0] 	CH0_PCM_RAM_write_address_buffer;

	wire [`DATA_WIDTH-1:0] 		CH0_PCM_RAM_write_data;
	wire [`ADDRESS_WIDTH-1:0] 	CH0_PCM_RAM_write_address;
	wire 								CH0_PCM_RAM_we;

	reg 								CH1_MAC_start;
	wire [`ADDRESS_WIDTH-1:0] 	CH1_MAC_RAM_read_address;
	wire [`DATA_WIDTH-1:0] 		CH1_MAC_RAM_read_data;
	wire 								CH1_MAC_RAM_read_en_MAC_clock;
	reg 								CH1_MAC_RAM_read_en_HUFF_clock;

	wire [`DATA_WIDTH-1:0] 		CH1_PCM_RAM_write_data;
	wire [`ADDRESS_WIDTH-1:0] 	CH1_PCM_RAM_write_address;
	wire 								CH1_PCM_RAM_we;

	wire 								MAC_done;

	// For AC97
	reg 								AC97_resetn;
	reg [1:0] 						sample_frequency;
	wire [`ADDRESS_WIDTH-1:0] 	PCM_RAM_read_address_bit_clock;
	reg [`ADDRESS_WIDTH-1:0] 	PCM_RAM_read_address_MAC_clock;
	wire [`DATA_WIDTH-1:0] 		CH0_PCM_RAM_read_data;
	wire [`DATA_WIDTH-1:0] 		CH1_PCM_RAM_read_data;
	wire 								AC97_source_select;
	
	reg [1:0] AC97_sample_frequency_cur_granule, AC97_sample_frequency_prv_granule;

	// Internal reg
	reg system_ready;

	clock_gen clock_module (
		.RESETN_I(Clock_resetn),
		.MASTER_CLOCK_I(MASTER_CLOCK_I),
//		.ALTERNATE_CLOCK_I(ALTERNATE_CLOCK_I),
		.AC97_BIT_CLOCK_I(AC97_BIT_CLOCK_I),
		.MASTER_CLOCK_O(master_clock),
		.MAC_CLOCK_O(MAC_clock),
		.HUFF_CLOCK_O(HUFF_clock),
		.AC97_BIT_CLOCK_O(AC97_bit_clock),
		.CLOCK_READY_O(Clock_ready),
		.MEM_CLK_FBOUT_P(MEM_CLK_FBOUT_P),
		.MEM_CLK_FBIN_P(MEM_CLK_FBIN_P),
		.clk_27mhz(clk_27mhz),
		.memory_27mhz(memory_27mhz)
	);
/*
	memctl32 memfill(
    .CLOCK_I(MAC_clock),
    .RESETN_I(ETH_resetn),
    .START_I(ETH_start),
    .ADDRESS_O(ETH_DP_ADDRESS),
    .DATA_O(ETH_DP_DATA),
    .WE_O(ETH_DP_WE),
    .EN_O(ETH_DP_EN),
    .DONE_O(ETH_done)
  );    
*/

assign ETH_DP_WE = ETH_DP_EN;
eth_bank_0 eth_bank_0_1(
   .resetn(ETH_resetn),
   .clk_27mhz(clk_27mhz),
   .memory_27mhz(memory_27mhz),
   .start_fill(ETH_start),
   .fill_done(ETH_done),
   .ext_busy(ETH_busy),
   .DP_address(ETH_DP_ADDRESS),
   .DP_enable(ETH_DP_EN),
   .DP_data(ETH_DP_DATA),
   .MEMORY_BANK0_CLK_P(MEMORY_BANK0_CLK_P),			.MEMORY_BANK1_CLK_P(MEMORY_BANK1_CLK_P),
   .MEMORY_BANK0_CLKEN_N(MEMORY_BANK0_CLKEN_N),		.MEMORY_BANK1_CLKEN_N(MEMORY_BANK1_CLKEN_N),
   .MEMORY_BANK0_WEN_N(MEMORY_BANK0_WEN_N),			.MEMORY_BANK1_WEN_N(MEMORY_BANK1_WEN_N),
   .MEMORY_BANK0_WENA_N(MEMORY_BANK0_WENA_N),		.MEMORY_BANK1_WENA_N(MEMORY_BANK1_WENA_N),
   .MEMORY_BANK0_WENB_N(MEMORY_BANK0_WENB_N),		.MEMORY_BANK1_WENB_N(MEMORY_BANK1_WENB_N),
   .MEMORY_BANK0_WENC_N(MEMORY_BANK0_WENC_N),		.MEMORY_BANK1_WENC_N(MEMORY_BANK1_WENC_N),
   .MEMORY_BANK0_WEND_N(MEMORY_BANK0_WEND_N),		.MEMORY_BANK1_WEND_N(MEMORY_BANK1_WEND_N),
   .MEMORY_BANK0_ADV_LD_N(MEMORY_BANK0_ADV_LD_N),	.MEMORY_BANK1_ADV_LD_N(MEMORY_BANK1_ADV_LD_N),
   .MEMORY_BANK0_OEN_N(MEMORY_BANK0_OEN_N),			.MEMORY_BANK1_OEN_N(MEMORY_BANK1_OEN_N),
   .MEMORY_BANK0_CEN_N(MEMORY_BANK0_CEN_N),			.MEMORY_BANK1_CEN_N(MEMORY_BANK1_CEN_N),
   .MEMORY_BANK0_ADDR_P(MEMORY_BANK0_ADDR_P),  		.MEMORY_BANK1_ADDR_P(MEMORY_BANK1_ADDR_P),  
   .MEMORY_BANK0_DATA_A_P(MEMORY_BANK0_DATA_A_P),	.MEMORY_BANK1_DATA_A_P(MEMORY_BANK1_DATA_A_P),
   .MEMORY_BANK0_DATA_B_P(MEMORY_BANK0_DATA_B_P),	.MEMORY_BANK1_DATA_B_P(MEMORY_BANK1_DATA_B_P),
   .MEMORY_BANK0_DATA_C_P(MEMORY_BANK0_DATA_C_P),	.MEMORY_BANK1_DATA_C_P(MEMORY_BANK1_DATA_C_P),
   .MEMORY_BANK0_DATA_D_P(MEMORY_BANK0_DATA_D_P),	.MEMORY_BANK1_DATA_D_P(MEMORY_BANK1_DATA_D_P),
   .TX_DATA_P(TX_DATA_P),
   .TX_ENABLE_P(TX_ENABLE_P),
   .TX_CLOCK_P(TX_CLOCK_P),
   .TX_ERROR_P(TX_ERROR_P),
   .ENET_SLEW_P(ENET_SLEW_P),
   .RX_DATA_P(RX_DATA_P),
   .RX_DATA_VALID_P(RX_DATA_VALID_P),
   .RX_ERROR_P(RX_ERROR_P),
   .RX_CLOCK_P(RX_CLOCK_P),
   .COLLISION_DETECTED_P(COLLISION_DETECTED_P),
   .CARRIER_SENSE_P(CARRIER_SENSE_P),
   .PAUSE_P(PAUSE_P),
   .MDIO_P(MDIO_P),
   .MDC_P(MDC_P),
   .MDINIT_N(MDINIT_N),
   .SSN_DATA_P(SSN_DATA_P),
	.range_end(ETH_range_end)
);

	// DP RAM between Huffman and Ethernet
  RAMB16_S18_S18 HUFF_DP (
		// output from Ethernet
    .DOA(),
    .DOPA(),
    .ADDRA(ETH_DP_ADDRESS),
    .CLKA(clk_27mhz),
    .DIA(ETH_DP_DATA),
    .DIPA(2'b00),
    .ENA(ETH_DP_EN),
    .SSRA(1'b0),
    .WEA(ETH_DP_WE),

		// input to HUFF	 
	 .DOB(HUFF_DP_DATA),
    .DOPB(),
    .ADDRB(HUFF_DP_ADDRESS),
    .CLKB(HUFF_clock),
    .DIB(16'h0000),
    .DIPB(2'b00),
    .ENB(1'b1),
    .SSRB(1'b0),
	 .WEB(1'b0)
  );
  /*
  wire skipframe4;
  wire [4:0] H_skipframe;
  wire [9:0] H_br_count;
  wire [8:0] H_big_values;
  wire [2:0] H_stage;
  wire [2:0] H_state;
  wire [2:0] H_step;
  wire [9:0] H_counter;
  wire [13:0] H_readptr;
  wire [13:0] H_newreadptr;
  wire [11:0] H_readbits;
  wire [15:0] H_data;
  wire H_source;
  wire [9:0] H_frame;
  wire [4:0] H_bits_processed;
  wire H_br_update;
  */
  wire H_gr;
  wire H_ch;

	// New huffman and requantizer with one RAM
	huffctl HUFFMANDECODER (
	/*
	 .skipframe4(skipframe4),
  	 .skipframe(H_skipframe),
	 .br_count(H_br_count),
    .big_values(H_big_values),
	 .stage(H_stage),
	 .state(H_state),
	 .step(H_step),
    .counter(H_counter),
	 .readptr(H_readptr),
	 .newreadptr(H_newreadptr),
	 .readbits(H_readbits),
	 .data(H_data),
	 .source(H_source),
	 .frame(H_frame),
	 .bits_processed(H_bits_processed),
    .br_update(H_br_update),
	*/
    .CLK_I(HUFF_clock),
    .RESETN_I(HUFF_resetn),
    
    .HUFF_DONE_O(HUFF_done),
    .MAC_READY_I(MAC_RAM_Idle),
    .DUAL_SELECT_I(PAL_NTSC),
    .WBBM_O(CH1_WBBM),
    .WBBML_O(CH0_WBBM),
    
    .GR_O(H_gr),
    .CH_O(H_ch),
    .NCH_O(),
    .SAMPLE_FREQUENCY_O(MP3_Info_Frequency),
    .MODE_O(MP3_Info_Mode),
    .MODE_EXTENSION_O(),
    
    .RAM_ADDRESS_O(Req_RAM_address),
    .RAM_WE_O(Req_RAM_we),
    .RAM_DATAIN_O(Req_RAM_write_data),
    .RAM_DATAOUT_I(Req_RAM_read_data),
    
    .DPR_ADDRESS_O(HUFF_DP_ADDRESS),
    .DPR_DATAOUT_I(HUFF_DP_DATA),
    
    .BR_ADDRESS_O(BR_RAM_address),
    .BR_DATAIN_O(BR_RAM_write_data),
    .BR_WE_O(BR_RAM_we),
    .BR_DATAOUT_I(BR_RAM_read_data),
    
    //.RIGHTCHANNEL_DATAIN_O(rightchannel_datain),
    //.RIGHTCHANNEL_ADDRESS_O(rightchannel_address),
    //.RIGHTCHANNEL_WE_O(rightchannel_we),
    //.LEFTCHANNEL_DATAIN_O(leftchannel_datain),
    //.LEFTCHANNEL_ADDRESS_O(leftchannel_address),
    //.LEFTCHANNEL_WE_O(leftchannel_we)
    .MAC_DATAIN_O(MAC_RAM_write_data),
    .MAC_ADDRESS_O(MAC_RAM_write_address),
    .MAC_LEFT_WE_O(CH0_MAC_RAM_we),
    .MAC_RIGHT_WE_O(CH1_MAC_RAM_we)
  );

  RAMB16_S18_S18 Req_RAM (
    .DOA(Req_RAM_read_data),
    .DOB(BR_RAM_read_data),
    .DOPA(),
    .DOPB(),
    .ADDRA(Req_RAM_address),
    .ADDRB({2'b11,BR_RAM_address}),
    .CLKA(HUFF_clock),
    .CLKB(HUFF_clock),
    .DIA(Req_RAM_write_data),
    .DIB(BR_RAM_write_data),
    .DIPA(2'b00),
    .DIPB(2'b00),
    .ENA(1'b1),
    .ENB(1'b1),
    .SSRA(1'b0),
    .SSRB(1'b0),
    .WEA(Req_RAM_we),
    .WEB(BR_RAM_we)
  );

	// DP RAM between Huffman and MAC
	RAMB16_S18_S18 CH0_MAC_RAM (
		// output from Huffman
    .DOA(),
    .DOPA(),
    .ADDRA(MAC_RAM_write_address),
    .CLKA(HUFF_clock),
    .DIA(MAC_RAM_write_data),
    .DIPA(2'b00),
    .ENA(1'b1),
    .SSRA(1'b0),
    .WEA(CH0_MAC_RAM_we),
	 
		// input to MAC
	 .DOB(CH0_MAC_RAM_read_data),
    .DOPB(),
    .ADDRB(CH0_MAC_RAM_read_address),
    .CLKB(MAC_clock),
    .DIB(16'h0000),
    .DIPB(2'b00),
    .ENB(CH0_MAC_RAM_read_en_MAC_clock),
    .SSRB(1'b0),
	 .WEB(1'b0)
  );

  RAMB16_S18_S18 CH1_MAC_RAM (
		// output from Huffman
    .DOA(),
    .DOPA(),
    .ADDRA(MAC_RAM_write_address),
    .CLKA(HUFF_clock),
    .DIA(MAC_RAM_write_data),
    .DIPA(2'b00),
    .ENA(1'b1),
    .SSRA(1'b0),
    .WEA(CH1_MAC_RAM_we),
	 
		// input to MAC
	 .DOB(CH1_MAC_RAM_read_data),
    .DOPB(),
    .ADDRB(CH1_MAC_RAM_read_address),
    .CLKB(MAC_clock),
    .DIB(16'h0000),
    .DIPB(2'b00),
    .ENB(CH1_MAC_RAM_read_en_MAC_clock),
    .SSRB(1'b0),
	 .WEB(1'b0)
  );

	// MAC
	mac_unit MAC_module (
		.MAC_CLOCK_I(MAC_clock), 
		.MAC_RESETN_I(MAC_resetn),

		.CH0_MAC_START_I(CH0_MAC_start),
		.CH0_MAC_BLOCK_I(MP3_Info_CH0_MAC_block_type_MAC_clock),
		.CH0_MAC_MIXED_I(MP3_Info_CH0_MAC_mixed_block_MAC_clock),
		.CH0_MAC_SWITCHING_I(MP3_Info_CH0_MAC_switching_MAC_clock),
		.CH0_MAC_RAM_DATA_I(CH0_MAC_RAM_read_data),
		.CH0_MAC_RAM_ADDRESS_O(CH0_MAC_RAM_read_address),
		.CH0_MAC_RAM_EN_O(CH0_MAC_RAM_read_en_MAC_clock),
		.CH0_PCM_RAM_DATA_O(CH0_PCM_RAM_write_data),
		.CH0_PCM_RAM_WE_O(CH0_PCM_RAM_we),
		.CH0_PCM_RAM_ADDRESS_O(CH0_PCM_RAM_write_address),

		.CH1_MAC_START_I(CH1_MAC_start),
		.CH1_MAC_BLOCK_I(MP3_Info_CH1_MAC_block_type_MAC_clock),
		.CH1_MAC_MIXED_I(MP3_Info_CH1_MAC_mixed_block_MAC_clock),
		.CH1_MAC_SWITCHING_I(MP3_Info_CH1_MAC_switching_MAC_clock),
		.CH1_MAC_RAM_DATA_I(CH1_MAC_RAM_read_data),
		.CH1_MAC_RAM_ADDRESS_O(CH1_MAC_RAM_read_address),
		.CH1_MAC_RAM_EN_O(CH1_MAC_RAM_read_en_MAC_clock),
		.CH1_PCM_RAM_DATA_O(CH1_PCM_RAM_write_data),
		.CH1_PCM_RAM_WE_O(CH1_PCM_RAM_we),
		.CH1_PCM_RAM_ADDRESS_O(CH1_PCM_RAM_write_address),

		.MAC_DONE_O(MAC_done)
	);

 	// DP RAM between MAC and AC97
	RAMB16_S18_S18 CH0_PCM_RAM (
		// output from MAC
    .DOA(),
    .DOPA(),
    .ADDRA(CH0_PCM_RAM_write_address),
    .CLKA(MAC_clock),
    .DIA(CH0_PCM_RAM_write_data),
    .DIPA(2'b00),
    .ENA(CH0_PCM_RAM_we),
    .SSRA(1'b0),
    .WEA(CH0_PCM_RAM_we),

		// input to AC97	 
	 .DOB(CH0_PCM_RAM_read_data),
    .DOPB(),
    .ADDRB(PCM_RAM_read_address_bit_clock),
    .CLKB(AC97_bit_clock),
    .DIB(16'h0000),
    .DIPB(2'b00),
    .ENB(1'b1),
    .SSRB(1'b0),
	 .WEB(1'b0)
  );

	RAMB16_S18_S18 CH1_PCM_RAM (
		// output from MAC
    .DOA(),
    .DOPA(),
    .ADDRA(CH1_PCM_RAM_write_address),
    .CLKA(MAC_clock),
    .DIA(CH1_PCM_RAM_write_data),
    .DIPA(2'b00),
    .ENA(CH1_PCM_RAM_we),
    .SSRA(1'b0),
    .WEA(CH1_PCM_RAM_we),

		// input to AC97	 
	 .DOB(CH1_PCM_RAM_read_data),
    .DOPB(),
    .ADDRB(PCM_RAM_read_address_bit_clock),
    .CLKB(AC97_bit_clock),
    .DIB(16'h0000),
    .DIPB(2'b00),
    .ENB(1'b1),
    .SSRB(1'b0),
	 .WEB(1'b0)
  );

	wire [15:0] AC97_CH0_read_data;
	wire [15:0] AC97_CH1_read_data;

	assign AC97_CH0_read_data = (MAC_state == 'd0) ? 16'h0000 : CH0_PCM_RAM_read_data;
	assign AC97_CH1_read_data = (MAC_state == 'd0) ? 16'h0000 : CH1_PCM_RAM_read_data;

	ac97_ctr ac97_module (
		.BIT_CLOCK_I(AC97_bit_clock),
		.SAMPLE_FREQUENCY_I(sample_frequency),
		.AC97_RESETN_I(AC97_resetn),
		.SOURCE_SELECT_I(AC97_source_select),
		.AC97_SYNCH_O(AC97_SYNCH_O),
		.AC97_DATA_IN_I(AC97_DATA_IN_I),
		.AC97_DATA_OUT_O(AC97_DATA_OUT_O),
		.AC97_BEEP_TONE_O(AC97_BEEP_TONE_O),
		.PCM_READ_ADDRESS_0(PCM_RAM_read_address_bit_clock),
		.CH0_PCM_DATA_I(AC97_CH0_read_data),
		.CH1_PCM_DATA_I(AC97_CH1_read_data),
		.STARTUP_O(STARTUP_O)
	); 	

	// For debugging
	parameter counter_width = 5;	
	parameter counter1_width = 5;	
	reg [counter_width-1:0] counter;
	reg [counter1_width-1:0] counter1;
				
	assign USER_LED0_O = counter[counter_width-1];
	assign USER_LED1_O = counter1[counter1_width-1];
	//assign USER_LED1_O = skipframe4;

	always @ (posedge CH0_MAC_start or negedge HUFF_resetn) begin
		if (HUFF_resetn == 1'b0) counter <= 0;
		else counter <= counter + 1;
	end

	always @ (posedge HUFF_done or negedge HUFF_resetn) begin
		if (HUFF_resetn == 1'b0) counter1 <= 0;
		else
			counter1 <= counter1 + 1;
	end

	always @ (posedge master_clock or negedge USER_INPUT0_I) begin
		if (USER_INPUT0_I == 1'b0) begin
			system_ready <= 1'b0;
			Clock_resetn <= 1'b0;
		end else begin
			system_ready <= (Clock_ready | system_ready);
			Clock_resetn <= 1'b1;
		end
	end

	always @ (system_ready or ETH_busy or USER_INPUT1_I or MP3_end_of_song) begin
		if (system_ready == 1'b0) begin
			// reset the whole FPGA while DCM is not locked
			AC97_resetn <= 1'b0;
			ETH_resetn <= 1'b0;
			MAC_resetn <= 1'b0;
		end else begin
			AC97_resetn <= USER_INPUT1_I;
			if (USER_INPUT1_I == 1'b1) begin
				if (MP3_end_of_song == 1'b0) begin
					if (ETH_busy == 1'b1) begin
//						AC97_resetn <= 1'b0;
						MAC_resetn <= 1'b0;
					end else begin
//						AC97_resetn <= USER_INPUT1_I;
						MAC_resetn <= USER_INPUT1_I;
					end
					ETH_resetn <= 1'b1;
				end else begin
//					AC97_resetn <= 1'b0;
					ETH_resetn <= 1'b0;
					MAC_resetn <= 1'b0;
				end
			end else begin
//				AC97_resetn <= 1'b0;
				ETH_resetn <= 1'b0;
				MAC_resetn <= 1'b0;
			end
		end
	end

	always @ (posedge MAC_clock) begin
		// buffer read address across bit clock domain
		PCM_RAM_read_address_MAC_clock <= PCM_RAM_read_address_bit_clock;
		MP3_Info_Mode_MAC_clock <= MP3_Info_Mode_buffer;

		if (PAL_NTSC == 1'b1 && MP3_Info_Mode_MAC_clock == 2'b10) begin
			MP3_Info_CH0_MAC_block_type_MAC_clock <= MP3_Info_CH1_MAC_block_type_buffer;
			MP3_Info_CH0_MAC_mixed_block_MAC_clock <= MP3_Info_CH1_MAC_mixed_block_buffer;
			MP3_Info_CH0_MAC_switching_MAC_clock <= MP3_Info_CH1_MAC_switching_buffer;
		end else begin
			MP3_Info_CH0_MAC_block_type_MAC_clock <= MP3_Info_CH0_MAC_block_type_buffer;
			MP3_Info_CH0_MAC_mixed_block_MAC_clock <= MP3_Info_CH0_MAC_mixed_block_buffer;
			MP3_Info_CH0_MAC_switching_MAC_clock <= MP3_Info_CH0_MAC_switching_buffer;
		end
		
		MP3_Info_CH1_MAC_block_type_MAC_clock <= MP3_Info_CH1_MAC_block_type_buffer;
		MP3_Info_CH1_MAC_mixed_block_MAC_clock <= MP3_Info_CH1_MAC_mixed_block_buffer;
		MP3_Info_CH1_MAC_switching_MAC_clock <= MP3_Info_CH1_MAC_switching_buffer;
	end

	always @ (posedge MAC_clock or negedge MAC_resetn) begin
		if (MAC_resetn == 1'b0) begin
			MAC_state <= 'd0;
			CH0_MAC_start <= 1'b0;
			CH1_MAC_start <= 1'b0;
			CH0_PCM_RAM_write_address_buffer <= 'd500;
		end else begin
			case (MAC_state) 
			'd0: begin
				if ((MAC_done == 1'b1) && (PCM_RAM_read_address_MAC_clock == CH0_PCM_RAM_write_address_buffer)) begin
					CH0_PCM_RAM_write_address_buffer <= 'd0;

					MAC_state <= 'd1;
				end		
			end
			'd1: begin
				// start MAC unit according to read address from AC97
				if ((CH0_MAC_start == 1'b0) && (MAC_done == 1'b1)
				 && (PCM_RAM_read_address_MAC_clock == CH0_PCM_RAM_write_address_buffer)) begin
					// start the MAC
					CH0_MAC_start <= 1'b1;
					if ((MP3_Info_Mode_MAC_clock == 2'b00) || (MP3_Info_Mode_MAC_clock == 2'b01)) CH1_MAC_start <= 1'b1;
					else CH1_MAC_start <= 1'b0;

					MAC_state <= 'd2;
				end
			end
			'd2: begin
				// de-assert the start signal
				if ((CH0_MAC_start == 1'b1) && (MAC_done == 1'b0)) begin
					CH0_MAC_start <= 1'b0;
					CH1_MAC_start <= 1'b0;

					CH0_PCM_RAM_write_address_buffer <= CH0_PCM_RAM_write_address;

					MAC_state <= 'd1;
				end
			end
			endcase
		end
	end

	reg [2:0] ETH_state;
	reg HUFF_address_msb_buffer;
	reg ETH_busy_MAC_clock;
	reg ETH_done_MAC_clock;
	reg ETH_range_end_MAC_clock;

	always @ (posedge MAC_clock) begin
		HUFF_address_msb_buffer <= HUFF_DP_ADDRESS[9];
		ETH_done_MAC_clock <= ETH_done;
		ETH_busy_MAC_clock <= ETH_busy;
		ETH_range_end_MAC_clock <= ETH_range_end;
	end

	always @ (posedge MAC_clock or negedge MAC_resetn) begin
		if (MAC_resetn == 1'b0) begin
			ETH_state <= 'd0;
			ETH_start <= 1'b0;
			HUFF_resetn <= 1'b0;
			MP3_end_of_song <= 1'b0;
		end else begin
			case (ETH_state) 
			'd0: begin
				// start ethernet at the same time as MAC
				if ((ETH_busy_MAC_clock == 1'b0) && (ETH_start == 1'b0) && (ETH_done_MAC_clock == 1'b1)
				 && (PCM_RAM_read_address_MAC_clock == CH0_PCM_RAM_write_address_buffer)) begin
					// start the Ethernet
					ETH_start <= 1'b1;

					ETH_state <= 'd1;
				end
			end
			'd1: begin
				// de-assert the start signal
				if ((ETH_start == 1'b1) && (ETH_done_MAC_clock == 1'b0)) begin
					ETH_start <= 1'b0;

					ETH_state <= 'd2;
				end
			end
			'd2: begin
				// start ethernet to fill the second half of the DP RAM
				if ((ETH_start == 1'b0) && (ETH_done_MAC_clock == 1'b1)) begin
					// start the Ethernet
					ETH_start <= 1'b1;

					ETH_state <= 'd3;

					HUFF_resetn <= 1'b1;
				end
			end
			'd3: begin
				// de-assert the start signal
				if ((ETH_start == 1'b1) && (ETH_done_MAC_clock == 1'b0)) begin
					ETH_start <= 1'b0;

					ETH_state <= 'd4;
				end
			end
			'd4: begin
				if ((HUFF_address_msb_buffer != HUFF_DP_ADDRESS[9]) && (ETH_done_MAC_clock == 1'b1)) begin
					if (ETH_range_end_MAC_clock == 1'b1) begin
						MP3_end_of_song <= 1'b1;
					end

					ETH_start <= 1'b1;

					ETH_state <= 'd3;
				end
			end
			endcase
		end
	end

	always @ (posedge HUFF_clock) begin
		CH0_MAC_RAM_read_en_HUFF_clock <= CH0_MAC_RAM_read_en_MAC_clock;
		CH1_MAC_RAM_read_en_HUFF_clock <= CH1_MAC_RAM_read_en_MAC_clock;
	end
	
	assign MAC_RAM_Idle = ~(CH0_MAC_RAM_read_en_HUFF_clock || CH1_MAC_RAM_read_en_HUFF_clock);

	// state machine for handshaking with huffman
	always @ (posedge HUFF_done or negedge MAC_resetn) begin
		if (MAC_resetn == 1'b0) begin
			MP3_Info_Frequency_buffer <= 2'd0;
			MP3_Info_Mode_buffer <= 2'd0;

			MP3_Info_CH0_MAC_block_type_buffer <= 2'b00;
			MP3_Info_CH0_MAC_mixed_block_buffer <= 1'b0;
			MP3_Info_CH0_MAC_switching_buffer <= 1'b0;

			MP3_Info_CH1_MAC_block_type_buffer <= 2'b00;
			MP3_Info_CH1_MAC_mixed_block_buffer <= 1'b0;
			MP3_Info_CH1_MAC_switching_buffer <= 1'b0;
		end else begin
			MP3_Info_Frequency_buffer <= MP3_Info_Frequency;
			MP3_Info_Mode_buffer <= MP3_Info_Mode;

			MP3_Info_CH0_MAC_block_type_buffer <= MP3_Info_CH0_MAC_block_type;
			MP3_Info_CH0_MAC_mixed_block_buffer <= MP3_Info_CH0_MAC_mixed_block;
			MP3_Info_CH0_MAC_switching_buffer <= MP3_Info_CH0_MAC_switching;

			MP3_Info_CH1_MAC_block_type_buffer <= MP3_Info_CH1_MAC_block_type;
			MP3_Info_CH1_MAC_mixed_block_buffer <= MP3_Info_CH1_MAC_mixed_block;
			MP3_Info_CH1_MAC_switching_buffer <= MP3_Info_CH1_MAC_switching;
		end
	end

	// For AC97
//	assign AC97_source_select = PAL_NTSC;
	assign AC97_source_select = 1'b1;

	always @ (posedge CH0_MAC_start or negedge MAC_resetn) begin
		if (MAC_resetn == 1'b0) begin
			AC97_sample_frequency_cur_granule <= 2'd0;
			AC97_sample_frequency_prv_granule <= 2'd0;
		end else begin
			AC97_sample_frequency_cur_granule <= MP3_Info_Frequency_buffer;
			AC97_sample_frequency_prv_granule <= AC97_sample_frequency_cur_granule;
		end
	end

	always @(posedge AC97_bit_clock or negedge AC97_resetn)
   begin
    	if (AC97_resetn == 1'b0) begin
			sample_frequency <= `FREQUENCY_48K;
		end
    	else begin
			if (S_VIDEO == 1'b1) sample_frequency <= AC97_sample_frequency_prv_granule;
			else sample_frequency <= `FREQUENCY_48K;
		end
	end
///////////////////////////////////////////////////////////////
/*
	 // for chip scope
    wire [35:0] control;
    wire [75:0] data;
    wire [7:0] trig0;
	 wire trigger_clock;

    assign trigger_clock = HUFF_clock;

  assign trig0[7] = H_stage;
  assign trig0[4] = MAC_RAM_Idle;
  assign trig0[3] = (H_frame == 9);
  assign trig0[2] = (H_frame == 12);
  assign trig0[1] = H_gr;//({1'b0,H_big_values} <= H_br_count);
  assign trig0[0] = (H_frame == 13);

  assign data[75:71] = H_bits_processed;
  assign data[70:61] = H_br_count;
  assign data[60:51] = H_counter;
  assign data[50:37] = H_newreadptr;
  assign data[36:27] = H_frame;
  assign data[26:24] = H_stage;
  assign data[23:21] = H_state;
  assign data[20:18] = H_step;
  assign data[17:6] = H_readbits;
  assign data[5:3] = 3'b0;
  assign data[2] = H_br_update;
  assign data[1] = H_gr;
  assign data[0] = H_ch;


  assign data[75:71] = H_skipframe;
  assign data[70:61] = H_br_count;
  assign data[60:52] = H_big_values;
  assign data[51:49] = H_stage;
  assign data[48:46] = H_state;
  assign data[45:43] = H_step;
  assign data[42:33] = H_counter;
  //assign data[32:19] = H_readptr;
  //assign data[18:5] = H_newreadptr;
  assign data[32:17] = H_data;
  assign data[16] = H_source;
  assign data[15:6] = H_frame;
  assign data[5:1] = H_bits_processed;
  assign data[0] = H_ch;


	 assign trigger_clock = MAC_clock;

	 assign trig0[7] = ETH_start;
	 assign trig0[6] = ETH_done;
	 assign trig0[5] = HUFF_done;
	 assign trig0[4] = HUFF_resetn;
	 assign trig0[3] = MAC_RAM_Idle;
	 assign trig0[2] = CH0_MAC_start;
	 assign trig0[1] = CH1_MAC_start;
	 assign trig0[0] = MAC_done;

	 assign data[75] = ETH_start;
	 assign data[74] = ETH_done;
	 assign data[73] = HUFF_resetn;
	 assign data[72] = HUFF_done;
	 assign data[71] = MAC_RAM_Idle;
	 assign data[70] = CH0_MAC_start;
	 assign data[69] = MP3_end_of_song;

	 assign data[68:59] = MAC_RAM_write_address;
	 assign data[58:43] = MAC_RAM_write_data;
	 assign data[42] = CH0_MAC_RAM_we;

	 assign data[41:32] = CH0_MAC_RAM_read_address;
	 assign data[31:16] = CH0_MAC_RAM_read_data;
	 assign data[15] = CH0_MAC_RAM_read_en_MAC_clock;

	 assign data[14:13] = MP3_Info_Frequency_buffer;
	 assign data[12:11] = MP3_Info_Mode_buffer;
	 assign data[10:9] = MP3_Info_CH0_MAC_block_type_buffer;
	 assign data[8] = MP3_Info_CH0_MAC_mixed_block_buffer;
	 assign data[7] = MP3_Info_CH0_MAC_switching_buffer;

	 assign data[6:5] = MP3_Info_Mode_MAC_clock;
	 assign data[4:3] = MP3_Info_CH0_MAC_block_type_MAC_clock;
	 assign data[2] = MP3_Info_CH0_MAC_mixed_block_MAC_clock;
	 assign data[1] = MP3_Info_CH0_MAC_switching_MAC_clock;
	 assign data[0] = MP3_end_of_song;


    icon i_icon
    (
     .control0(control)
    );

    ila i_ila
    (
      .control(control),
      .clk(trigger_clock),
      .data(data),
      .trig0(trig0)
    );

endmodule

module icon 
  (
      control0
  );
  output [35:0] control0;
endmodule

module ila
  (
    control,
    clk,
    data,
    trig0
  );
  input [35:0] control;
  input clk;
  input [75:0] data;
  input [7:0] trig0;

*/
///////////////////////////////////////////////////////////////////////
endmodule


// Inputs to the design

// USER_INIPUT0 = Async global reset

// USER_INIPUT1 = reset for mac_unit
// PAL_SYNC = Dual channel selection

// S_VIDEO = Sampling frequency
