// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================


`timescale 1 ns/ 100 ps

`include "defines.v"

`define EOF 32'hFFFF_FFFF
`define NULL 0
`define MAX_LINE_LENGTH 1000

`define NUM_MAC_SAMPLE 'd576
`define NUM_PCM_SAMPLE 'd576
`define NUM_OF_FRAME 'd100
`define NUM_OF_GRANULE 'd2
`define NUM_OF_CHANNEL 'd2

module mp3_top_tb_mp3_v_tf();

// DATE:     10:46:06 07/25/2005 
// MODULE:   mp3_top
// DESIGN:   mp3_top
// FILENAME: tb_mp3.v
// PROJECT:  mp3_top
// VERSION:  


// Inputs
    reg MASTER_CLOCK_I;
    reg AC97_BIT_CLOCK_I;
    reg AC97_DATA_IN_I;
    reg USER_INPUT0_I;
    reg USER_INPUT1_I;
    reg PAL_NTSC;
    reg S_VIDEO;


// Outputs
    wire AC97_SYNCH_O;
    wire AC97_DATA_OUT_O;
    wire AC97_BEEP_TONE_O;
    wire USER_LED0_O;
    wire USER_LED1_O;
    wire STARTUP_O;

// Bidirs


// Instantiate the UUT
    mp3_top uut (
        .MASTER_CLOCK_I(MASTER_CLOCK_I), 
        .AC97_BIT_CLOCK_I(AC97_BIT_CLOCK_I), 
        .AC97_SYNCH_O(AC97_SYNCH_O), 
        .AC97_DATA_IN_I(AC97_DATA_IN_I), 
        .AC97_DATA_OUT_O(AC97_DATA_OUT_O), 
        .AC97_BEEP_TONE_O(AC97_BEEP_TONE_O), 
        .USER_INPUT0_I(USER_INPUT0_I), 
        .USER_INPUT1_I(USER_INPUT1_I), 
        .PAL_NTSC(PAL_NTSC), 
        .S_VIDEO(S_VIDEO), 
        .USER_LED0_O(USER_LED0_O), 
        .USER_LED1_O(USER_LED1_O), 
        .STARTUP_O(STARTUP_O)
        );


// the clock signals
parameter master_cycle = 18.5;

always begin
	#(master_cycle);
	MASTER_CLOCK_I = 1'b0;
	#(master_cycle);
	MASTER_CLOCK_I = 1'b1;
end

always begin
	#(master_cycle*2.197265625);
	AC97_BIT_CLOCK_I = 1'b0;
	#(master_cycle*2.197265625);
	AC97_BIT_CLOCK_I = 1'b1;
end

task master_reset;
// task for generating master reset for two clock cycles
begin
	USER_INPUT0_I = 1'b1;

	wait (MASTER_CLOCK_I !== 1'bx);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT0_I = 1'b0;
	@ (posedge MASTER_CLOCK_I);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT0_I = 1'b1;
end
endtask

task other_reset;
// task for generating other reset for two clock cycles
begin
	USER_INPUT1_I = 1'b1;

	wait (MASTER_CLOCK_I !== 1'bx);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT1_I = 1'b0;
	@ (posedge MASTER_CLOCK_I);
	@ (posedge MASTER_CLOCK_I);
	USER_INPUT1_I = 1'b1;
end
endtask

// Initialize Inputs
integer i;

// file pointers
integer control_fp;
integer pcm_in_fp, pcm_out_fp;
integer mac_in_fp, mac_out_fp;
integer alias_out_fp;
integer mdct_out_fp, mdct_in_fp;
//integer wr_frame, wr_granule, wr_channel, wr_sample;

// global variables
integer frame, granule, channel, sample;

// for reading data from files										
reg [`DATA_WIDTH-1:0] pcm_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][`NUM_PCM_SAMPLE-1:0];

integer unmatch_count [`NUM_OF_CHANNEL-1:0];

task read_pcm_data;
// task for reading data from file for verification
integer frame, granule, channel, sample;
integer pcm_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	pcm_data_fp = $fopen("TXT/debug_from_subsyn_16.txt", "r");

	if (pcm_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(pcm_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, pcm_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, pcm_data_fp);
         file_r = $fscanf(pcm_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

			pcm_data[frame][granule][channel][sample_num] = sample_hex;
			sample = sample + 1;
			if (sample_num == 575) begin
				$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
				sample = 0;
				if (channel == 0) begin
					channel = 1;
				end
				else begin
					if (granule == 0) begin
						granule = 1;
					end else begin
						granule = 0;
						frame = frame + 1;
					end
					channel = 0;
				end
			end
		end
		file_c = $fgetc(pcm_data_fp);
	end

	$fclose(pcm_data_fp);

	$display("Finish reading pcm data file...");
end
endtask

reg [`DATA_WIDTH-1:0] requantizer_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][1023:0];
reg [`DATA_WIDTH-1:0] requantizer_block_type [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];
reg [1:0] requantizer_mode [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0];
reg [1:0] requantizer_mode_ext [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0];
reg [1:0] requantizer_frequency [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0];

task read_requantizer_data;
// task for reading anti-alias data from file for verification
integer frame, granule;
integer address;
integer requantizer_data_fp;
integer requantizer_data_CH0_fp;
integer requantizer_data_CH1_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [7:0] sample_hex1, sample_hex2;

begin
	frame = 0;
	granule = 0;
	channel = 0;

	requantizer_data_CH0_fp = $fopen("TXT/left.txt", "r");
	requantizer_data_CH1_fp = $fopen("TXT/right.txt", "r");
	
	if ((requantizer_data_CH0_fp == `NULL) || (requantizer_data_CH0_fp == `NULL)) begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	for (channel = 0; channel < 2; channel = channel + 1) begin
		if (channel == 0) requantizer_data_fp = requantizer_data_CH0_fp;
		else requantizer_data_fp = requantizer_data_CH1_fp;

		file_c = $fgetc(requantizer_data_fp);

		address = -2;
		frame = 0;
		granule = 0;

		for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
			/* Check the first character for comment */
	      if (file_c == "/") begin
				file_r = $fgets(line, requantizer_data_fp);
	      end else begin
		  		// Push the character back to the file then read the next time
	         file_r = $ungetc(file_c, requantizer_data_fp);
	         file_r = $fscanf(requantizer_data_fp,"%h\n", sample_hex1);
		      file_r = $fscanf(requantizer_data_fp,"%h\n", sample_hex2);

				if (address > -1) begin
					requantizer_data[frame][granule][channel][address] = {sample_hex1, sample_hex2};
//					$write("fr=%d gr=%d ch=%d address=%d data=%h\n", frame, granule, channel, address, requantizer_data[frame][granule][channel][address]);
				end else begin
					if (address == -1) begin
						if (channel == 0) begin
							requantizer_mode[frame][granule] = sample_hex1[1:0];
							requantizer_mode_ext[frame][granule] = sample_hex2[1:0];
//							$write("\t\t\t\t\t\t\tfr=%d gr=%d address=%d mode=%d mode_ext=%d\n", frame, granule, address, requantizer_mode[frame][granule], requantizer_mode_ext[frame][granule]);
						end
					end else begin
						if (channel == 0) begin
							requantizer_frequency[frame][granule] = sample_hex2[1:0];
//							$write("\t\t\t\t\t\t\tfr=%d gr=%d address=%d frequency=%d\n", frame, granule, address, requantizer_frequency[frame][granule]);
						end
					end
				end
			
				if (address == 'd592 && granule == 'd0) begin
					requantizer_block_type[frame][granule][channel] = {14'h0, requantizer_data[frame][granule][channel][address][9:8]};
				end

				if (address == 'd596 && granule == 'd1) begin
					requantizer_block_type[frame][granule][channel] = {14'h0, requantizer_data[frame][granule][channel][address][9:8]};
				end

				address = address + 1;

				if (address == 1024) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, address);
					address = -2;
					if (granule == 0) begin
						granule = 1;
					end else begin
						granule = 0;
						frame = frame + 1;
					end
				end
			end
			file_c = $fgetc(requantizer_data_fp);
		end
	end

	$fclose(requantizer_data_CH0_fp);
	$fclose(requantizer_data_CH1_fp);

	$display("Finish reading requantizer data file...");
end
endtask

reg [`DATA_WIDTH-1:0] alias_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][`NUM_PCM_SAMPLE-1:0];
reg [`DATA_WIDTH-1:0] alias_block_type [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];

task read_alias_data;
// task for reading anti-alias data from file for verification
integer frame, granule, channel, sample;
integer alias_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	alias_data_fp = $fopen("TXT/debug_alias_input_16.txt", "r");

	if (alias_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(alias_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, alias_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, alias_data_fp);
         file_r = $fscanf(alias_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

			if (sample_num == -1) begin
				alias_block_type[frame][granule][channel] = sample_hex;
//				$write("fr=%d gr=%d ch=%d block_type=%d\n", frame, granule, channel, alias_block_type[frame][granule][channel]);
			end else begin
				alias_data[frame][granule][channel][sample_num] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d sample_num=%d data=%h\n", frame, granule, channel, sample, sample_num, alias_data[frame][granule][channel][sample_num]);
				sample = sample + 1;
				if (sample_num == 575) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
			end
		end
		file_c = $fgetc(alias_data_fp);
	end

	$fclose(alias_data_fp);

	$display("Finish reading alias data file...");
end
endtask


integer log_fp;

task check_data;
input frame, granule, channel, sample;
integer frame, granule, channel, sample;
input got_data;
reg [`DATA_WIDTH-1:0] got_data;
begin
	if (got_data != pcm_data[frame][granule][channel][sample]) begin
		log_fp = $fopen("TXT/data_log.txt", "a");
		$fwrite(log_fp, "Data unmatched at fr=%d gr=%d ch=%d sample=%d got=%h expected=%h\n", frame, granule, channel, sample, got_data, pcm_data[frame][granule][channel][sample]);
		//$write("Data unmatched at fr=%d gr=%d ch=%d sample=%d got=%h expected=%h\n", frame, granule, channel, sample, got_data, pcm_data[frame][granule][channel][sample]);
		$fclose(log_fp);
		unmatch_count[channel] = unmatch_count[channel] + 1;
	end
end
endtask

        initial begin
				// read expected data
				read_requantizer_data;
				read_alias_data;
				read_pcm_data;

				frame = -2;
				channel = 0;
				granule = 0;
				sample = 0;
  				i = 0;
				unmatch_count[0] = 0;
				unmatch_count[1] = 0;

            AC97_DATA_IN_I = 1;
            USER_INPUT0_I = 1;
            USER_INPUT1_I = 1;
            PAL_NTSC = 1;
            S_VIDEO = 1;

				$display("INFO: Simulation started...");

				// Generate reset signals
				master_reset;
				other_reset;

				$display("INFO: Reset signals generated...");

				control_fp = $fopen("TXT/control.txt", "w");
				pcm_out_fp = $fopen("TXT/pcm_out_temp.txt", "w");
				pcm_in_fp = $fopen("TXT/pcm_in_temp.txt", "w");
				mac_out_fp = $fopen("TXT/mac_out_temp.txt", "w");
				mac_in_fp = $fopen("TXT/mac_in_temp.txt", "w");
				alias_out_fp = $fopen("TXT/alias_out_temp.txt", "w");
				mdct_in_fp = $fopen("TXT/mdct_in_temp.txt", "w");
				mdct_out_fp = $fopen("TXT/mdct_out_temp.txt", "w");
				log_fp = $fopen("TXT/data_log.txt", "w");
				 
				$fclose(control_fp);
				$fclose(pcm_out_fp);
				$fclose(pcm_in_fp);
				$fclose(mac_out_fp);
				$fclose(mac_in_fp);
				$fclose(alias_out_fp);
				$fclose(mdct_in_fp);
				$fclose(mdct_out_fp);
				$fclose(log_fp);

				$display("INFO: Program started...");
	     end

		  always @ (negedge uut.ETH_done) begin
			$write("ETH started...\n");
			$write("\t\t\t\tAddress[9]=%d buffer=%d...\n", uut.HUFF_DP_ADDRESS[9], uut.HUFF_address_msb_buffer);
		  end

		  always @ (posedge uut.ETH_done) begin
		  	$write("ETH finished...\n");
		  end

		  always @ (posedge uut.MP3_Frame_done) begin
		  	$write("MP3 Frame finished...\n");
		  end

  		  always @ (negedge uut.MP3_Frame_done) begin
		  	$write("MP3 Frame started...\n");
		  end

  		  always @ (posedge uut.HUFF_done) begin
		  	$write("Huffman finish...\n");
		  end

		  always @ (negedge uut.HUFF_done) begin
		  	$write("Huffman started...\n");
		  end

  		  always @ (posedge uut.Requantizer_done) begin
		  	$write("Requantizer finish...\n");
		  end

		  always @ (negedge uut.Requantizer_done) begin
		  	$write("Requantizer started...\n");
			$write("\t\t\t\t\t\t\t\t\t\t\tgranule_flag=%d\n", uut.instance_Requantizer.granule_flag);
		  end

		  always @ (posedge uut.instance_Requantizer.Stereo_dec_flag) begin
		  	$write("Stereo decoding started...\n");
		  end

		  always @ (negedge uut.Stereo_start) begin
		  	$write("Stereo finish\n");
		  end
			integer realframe;
			initial begin
				realframe = 40;
			end

			always @ (posedge uut.HUFF_clock) begin
				if (uut.HUFFMANDECODER.state == 6)
				begin
					realframe = realframe + 1;
					$write("===========================================================\t\t\t\t\tcurrent frame: %d\n",realframe);
				end
			end

		  always @ (posedge uut.HUFF_clock) begin
			  if (uut.HUFFMANDECODER.state == 11) begin
			    $write("HUFF write %d: \tglobal_gain = %d \tscalefac_compress = %d \tWSF = %d\n",
				   uut.HUFFMANDECODER.ram0_address,
				   uut.HUFFMANDECODER.ram0_datain[15:8],
				   uut.HUFFMANDECODER.ram0_datain[7:4],
					uut.HUFFMANDECODER.ram0_datain[3]
				 );
			    if ( uut.HUFFMANDECODER.ram0_datain[3] == 1)
				   $write("Block_type = %d \tMBF = %d\n",
					  uut.HUFFMANDECODER.ram0_datain[2:1],
					  uut.HUFFMANDECODER.ram0_datain[0]
				   );
			  end
		  end

		  always @ (posedge uut.HUFF_clock) begin
			if (uut.instance_Requantizer.counter == 576 + 2 + 1 || uut.instance_Requantizer.counter == 584 + 2 + 1)
				$write("RAM global_gain = %d \tWSF = %d \tBlock_type = %d \tMBF = %d\n",
					uut.instance_Requantizer.REQ_DATA_I[15:8],
					uut.instance_Requantizer.REQ_DATA_I[3],
					uut.instance_Requantizer.REQ_DATA_I[2:1],
					uut.instance_Requantizer.REQ_DATA_I[0]
				);
		  end

		  always @ (posedge uut.HUFF_clock) begin
		  		if (uut.instance_Requantizer.state == `State_ROM_Addr_comp && uut.instance_Requantizer.counter == 0) begin
					$write("global_gain=%d block_type=%d scalefac_scale=%d preflag=%d	mixed_block_flag=%d window_switching_flag=%d zeros_pos_start=%d\n",
						uut.instance_Requantizer.global_gain,
						uut.instance_Requantizer.block_type,
						uut.instance_Requantizer.scalefac_scale,
						uut.instance_Requantizer.preflag,
						uut.instance_Requantizer.mixed_block_flag,
				      uut.instance_Requantizer.window_switching_flag,
						uut.instance_Requantizer.zeros_pos_start
					);
				end
		  end

				 /*
		// for testing requantizer
		always @ (posedge uut.HUFF_clock) begin
			if (MEM_frame >= 'd5) begin
		   if (uut.instance_Requantizer.state == `State_reorder && uut.instance_Requantizer.counter !=576)begin
				 $write("count=%d  add0=%d add1=%d coeff=%h sca_fac=%d ch0_I=%h ch1_I=%h ch0_d=%h ch1_d=%h	\n",
				 uut.instance_Requantizer.counter,
				 uut.instance_Requantizer.DP_CH0_ADDRESS_O,
				 uut.instance_Requantizer.DP_CH1_ADDRESS_O,
				 uut.instance_Requantizer.Mult_in2,
				 uut.instance_Requantizer.scalefac_L,
				 uut.instance_Requantizer.REQ_CH0_DATA_I,
				 uut.instance_Requantizer.REQ_CH1_DATA_I,
				 uut.instance_Requantizer.DP_CH0_DATA_O,  
				 uut.instance_Requantizer.DP_CH1_DATA_O );
			end
			end
		end


			 always @(posedge uut.HUFF_clock)
			  if(uut.instance_Requantizer.state ==`State_value_Computation && uut.instance_Requantizer.Stereo_dec_flag==1'b0)
				  begin	
				  $display(" counter=%d\t  gain_exp=%h\t window=%d\t gain_index=%h in1=%d\t in2=%d\t mult_out=%d\t add_ch0=%d\t ch0=%h\t\t\t add_ch1=%d\t ch1=%h",
				  //$display(" i=%d\t add_ch0=%d\t ch0=%h add_ch1=%d\t ch1=%h",	
							uut.instance_Requantizer.counter,
							uut.instance_Requantizer.gain_exp,
							uut.instance_Requantizer.window, 
							uut.instance_Requantizer.gain_index,
							uut.instance_Requantizer.Mult_in1,
							uut.instance_Requantizer.Mult_in2,
							uut.instance_Requantizer.mult_out,
							uut.instance_Requantizer.REQ_CH0_ADDRESS_O,  				 
							uut.instance_Requantizer.REQ_CH0_DATA_O,
							uut.instance_Requantizer.REQ_CH1_ADDRESS_O,  				 
							uut.instance_Requantizer.REQ_CH1_DATA_O);
				  end

			  */

			// for behavioral

		  
		  always @ (posedge AC97_SYNCH_O) begin
		  		// Print read PCM sample
		  		if (i > 15) begin
					pcm_out_fp = $fopen("TXT/pcm_out_temp.txt", "a");
					//$fwrite(pcm_out_fp, "READ_PCM: (%d, %d) ra=%d, ld=%h (%d), rd=%h (%d)\n", frame, granule, uut.ac97_module.sample_counter, uut.ac97_module.CH0_PCM_DATA_I, uut.ac97_module.CH0_PCM_DATA_I, uut.ac97_module.CH1_PCM_DATA_I, uut.ac97_module.CH1_PCM_DATA_I);
					$fclose(pcm_out_fp);

					if (sample % 100 == 0) $write("Sample read: %d\n", sample);

					//if (frame >= 0) begin
					//	check_data(frame, granule, 0, sample, uut.ac97_module.CH0_PCM_DATA_I);
					//	check_data(frame, granule, 1, sample, uut.ac97_module.CH1_PCM_DATA_I);
					//end

					sample = sample+1;

					if (sample == `NUM_PCM_SAMPLE) begin
						//$write("AC97 finished reading (fr=%d gr=%d) from PCM_RAM...\n", frame, granule);
						if (frame >= 0) begin
							//$write("Last left value in frame: got=%h expected=%h\n", uut.ac97_module.CH0_PCM_DATA_I, pcm_data[frame][granule][0][sample-1]);
							//$write("Last right value in frame: got=%h expected=%h\n", uut.ac97_module.CH1_PCM_DATA_I, pcm_data[frame][granule][1][sample-1]);
							//$write("Channel 0: Unmatch count=%d\n", unmatch_count[0]);
							//$write("Channel 1: Unmatch count=%d\n", unmatch_count[1]);
							if (granule == 0) granule = 1;
							else begin
								granule = 0;
								frame = frame + 1;
							end
						end else
						frame = frame + 1;
						sample = 0;
					end

					if (frame == `NUM_OF_FRAME) begin
						# 1000;
						log_fp = $fopen("TXT/data_log.txt", "a");
						$fwrite(log_fp, "\n\nTotal unmatch count: ch0=%d ch1=%d\n", unmatch_count[0], unmatch_count[1]);
						$write("\n\nTotal unmatch count: ch0=%d ch1=%d\n", unmatch_count[0], unmatch_count[1]);
						$fclose(log_fp);
						$stop(2);
					end
				end
		  end
		  
		  always @ (posedge AC97_SYNCH_O) begin
		  		if (i > 15) #10;
				else 
				i = i+1;
		
//					$write("INFO: %d: %d\n", i, uut.MAC_module_MAC_RAM_ADDRESS_O);
				#10;
		  end
/*
			always @ (posedge uut.MAC_clock) begin
			if (uut.CH0_MAC_start == 1'b1 && uut.MAC_done == 1'b0) begin
				$write("LEFT MAC started...\n");
				$write("\t\t\t\t\t\t\t\t\t\tstate=%d\n", uut.MAC_module.CH0_state);
				$write("\t\t\t\t\t\t\t\t\t\tblock_type=%d\n", uut.MAC_module.CH0_MAC_BLOCK_I);
				$write("\t\t\t\t\t\t\t\t\t\thold=%d\n", uut.MAC_module.CH0_hold);
				$write("\t\t\t\t\t\t\t\t\t\tcounter=%d\n", uut.MAC_module.CH0_counter);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_pointer=%d\n", uut.MAC_module.CH0_aic_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_offset=%d\n", uut.MAC_module.CH0_aic_read_offset);
				$write("\t\t\t\t\t\t\t\t\t\ticg_read_pointer=%d\n", uut.MAC_module.CH0_icg_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_write_pointer=%d\n", uut.MAC_module.CH0_antialias_write_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_read_pointer=%d\n", uut.MAC_module.CH0_antialias_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tbackup_pointer=%d\n", uut.MAC_module.CH0_backup_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tMAC_RAM_pointer=%d\n", uut.MAC_module.CH0_MAC_RAM_pointer);
				$write("\n");			
			end
			if (uut.CH1_MAC_start == 1'b1 && uut.MAC_done == 1'b0) begin
				$write("RIGHT MAC started...\n");
				$write("\t\t\t\t\t\t\t\t\t\tstate=%d\n", uut.MAC_module.CH1_state);
				$write("\t\t\t\t\t\t\t\t\t\tblock_type=%d\n", uut.MAC_module.CH1_MAC_BLOCK_I);
				$write("\t\t\t\t\t\t\t\t\t\thold=%d\n", uut.MAC_module.CH1_hold);
				$write("\t\t\t\t\t\t\t\t\t\tcounter=%d\n", uut.MAC_module.CH1_counter);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_pointer=%d\n", uut.MAC_module.CH1_aic_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\taic_read_offset=%d\n", uut.MAC_module.CH1_aic_read_offset);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_write_pointer=%d\n", uut.MAC_module.CH1_antialias_write_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tantialias_read_pointer=%d\n", uut.MAC_module.CH1_antialias_read_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tbackup_pointer=%d\n", uut.MAC_module.CH1_backup_pointer);
				$write("\t\t\t\t\t\t\t\t\t\tMAC_RAM_pointer=%d\n", uut.MAC_module.CH1_MAC_RAM_pointer);
			end
			if (uut.CH0_MAC_start == 1'b1) begin
				$write("\n");			
				$write("\t\t\t\t\t\t\t\t\t\tPCM_RAM_pointer=%d\n", uut.MAC_module.PCM_RAM_pointer);
			end
		  end


		  always @ (posedge uut.MAC_done) begin
		  	$write("MAC finished...wab=%d\n", uut.CH0_PCM_RAM_write_address_buffer);
		  end
*/
/*
		  always @ (posedge uut.MAC_clock) begin
		  		// Control signals
				control_fp = $fopen("TXT/control.txt", "a");
				$fwrite(control_fp, "MEM_start=%d, MEM_done=%d, MAC_RAM_read_en=%d, MAC_RAM_write_address=%d, MAC_block_type=%d, ",
							uut.MEM_start, 
							uut.MEM_done, 
							uut.MAC_RAM_read_en,
							uut.MAC_RAM_write_address,
							uut.MAC_block_type
							);
				$fwrite(control_fp, "MAC_start=%d, MAC_done=%d, PCM_RAM_read_address_buffer=%d, PCM_RAM_write_address_buffer=%d\n", 
							uut.MAC_start, 
							uut.MAC_done,
							uut.PCM_RAM_read_address_buffer,
							uut.PCM_RAM_write_address_buffer
							);

				$fclose(control_fp);
		  end
*/

// for writing data to the DP RAM (MAC RAM)
			integer alias_state;
			integer alias_sb_count;
			integer alias_counter;
			integer left_alias_frame;
			integer left_alias_granule;
			integer left_alias_sample;
			integer right_alias_frame;
			integer right_alias_granule;
			integer right_alias_sample;
			reg [`DATA_WIDTH-1:0] CH0_ram_data [1023:0];
			reg [`DATA_WIDTH-1:0] CH1_ram_data [1023:0];
			reg Stereo_start_buf;
			reg [1:0] MP3_Info_CH0_block_type, MP3_Info_CH1_block_type;
	
			always @ (posedge uut.MAC_clock) begin
				Stereo_start_buf <= uut.instance_Requantizer.Stereo_dec_flag;
			end

			always @ (posedge uut.MAC_clock or negedge uut.ETH_resetn) begin
		  		if (uut.ETH_resetn == 0) begin
					alias_state <= 'd0;
					alias_sb_count <= 'd0;
					alias_counter <= 'd0;
					left_alias_frame <= 'd0;
					left_alias_granule <= 'd0;
					left_alias_sample <= 'd0;
					right_alias_frame <= 'd0;
					right_alias_granule <= 'd0;
					right_alias_sample <= 'd0;				
					MP3_Info_CH0_block_type <= 'd0;
					MP3_Info_CH1_block_type <= 'd0;
				end else begin
					if (uut.instance_Requantizer.Stereo_dec_flag == 1'b1 && Stereo_start_buf == 1'b0) begin
						MP3_Info_CH0_block_type <= alias_block_type[left_alias_frame][left_alias_granule][0];
						MP3_Info_CH1_block_type <= alias_block_type[right_alias_frame][right_alias_granule][1];

						alias_state <= 'd1;
						alias_sb_count <= 'd0;
						alias_counter <= 'd0;
						left_alias_sample <= 'd0;
						right_alias_sample <= 'd0;
					end else begin
						if (alias_counter + alias_sb_count * 32 == 'd1023) begin
							alias_state <= 'd0;
							alias_sb_count <= 'd0;
							alias_counter <= 'd0;
							left_alias_sample <= 'd0;
							right_alias_sample <= 'd0;
						end
					end

					if (alias_state == 'd0) begin
						alias_counter <= 'd0;
						alias_sb_count <= 'd0;
					end else begin
						if (alias_counter < 'd31) begin
							alias_counter <= alias_counter + 1;
						end else begin
							alias_sb_count <= alias_sb_count + 1;
							alias_counter <= 'd0;
						end
					end

					if (alias_state == 'd1) begin
						if (MP3_Info_CH0_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								if (left_alias_sample == `NUM_MAC_SAMPLE - 1) begin
									if (left_alias_granule == 'd0) left_alias_granule <= 'd1;
									else begin
										left_alias_frame <= left_alias_frame + 1;
										left_alias_granule <= 'd0;
										left_alias_sample <= 'd0;
									end
								end else begin
									left_alias_sample <= left_alias_sample + 1;
								end
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									if (left_alias_sample == `NUM_MAC_SAMPLE-1) begin
										if (left_alias_granule == 'd0) left_alias_granule <= 'd1;
										else begin
											left_alias_frame <= left_alias_frame + 1;
											left_alias_granule <= 'd0;
											left_alias_sample <= 'd0;
										end
									end else begin
										left_alias_sample <= left_alias_sample + 1;
									end
								end
							end
						end
					end

					if (alias_state == 'd1) begin
						if (MP3_Info_CH1_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								if (right_alias_sample == `NUM_MAC_SAMPLE - 1) begin
									if (right_alias_granule == 'd0) right_alias_granule <= 'd1;
									else begin
										right_alias_frame <= right_alias_frame + 1;
										right_alias_granule <= 'd0;
										right_alias_sample <= 'd0;
									end
								end else begin
									right_alias_sample <= right_alias_sample + 1;
								end
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									if (right_alias_sample == `NUM_MAC_SAMPLE-1) begin
										if (right_alias_granule == 'd0) right_alias_granule <= 'd1;
										else begin
											right_alias_frame <= right_alias_frame + 1;
											right_alias_granule <= 'd0;
											right_alias_sample <= 'd0;
										end
									end else begin
										right_alias_sample <= right_alias_sample + 1;
									end
								end
							end
						end
					end
		
					if (alias_state == 'd1) begin
						if (MP3_Info_CH0_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin								
								CH0_ram_data[alias_counter + alias_sb_count * 32]  <= alias_data[left_alias_frame][left_alias_granule][0][(left_alias_sample + 9) % `NUM_MAC_SAMPLE];
/*
								$write("l: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH0_block_type,
											alias_sb_count, 
											alias_counter,
											left_alias_frame, 
											left_alias_granule, 
											left_alias_sample, 
											(left_alias_sample + 9) % `NUM_MAC_SAMPLE,
											alias_counter + alias_sb_count * 32, 
											alias_data[left_alias_frame][left_alias_granule][0][(left_alias_sample + 9) % `NUM_MAC_SAMPLE]
											);
*/
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									CH0_ram_data[alias_counter + alias_sb_count * 32] <= alias_data[left_alias_frame][left_alias_granule][0][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3];
/*
									$write("l: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH0_block_type,
											alias_sb_count, 
											alias_counter,
											left_alias_frame, 
											left_alias_granule, 
											left_alias_sample, 
											(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3,
											alias_counter + alias_sb_count * 32, 
											alias_data[left_alias_frame][left_alias_granule][0][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3]
											);
*/
								end
							end
						end
					end

					if (alias_state == 'd1) begin
						if (MP3_Info_CH1_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								CH1_ram_data[alias_counter + alias_sb_count * 32] <= alias_data[right_alias_frame][right_alias_granule][1][(right_alias_sample + 9) % `NUM_MAC_SAMPLE];
/*
								$write("r: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH1_block_type,
											alias_sb_count, 
											alias_counter,
											right_alias_frame, 
											right_alias_granule, 
											right_alias_sample, 
											(right_alias_sample + 9) % `NUM_MAC_SAMPLE,
											alias_counter + alias_sb_count * 32, 
											alias_data[right_alias_frame][right_alias_granule][1][(right_alias_sample + 9) % `NUM_MAC_SAMPLE]
											);
*/
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									CH1_ram_data[alias_counter + alias_sb_count * 32] <= alias_data[right_alias_frame][right_alias_granule][1][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3];
/*
									$write("r: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH1_block_type,
											alias_sb_count, 
											alias_counter,
											right_alias_frame, 
											right_alias_granule, 
											right_alias_sample, 
											(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3,
											alias_counter + alias_sb_count * 32, 
											alias_data[right_alias_frame][right_alias_granule][1][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3]
											);
*/
								end
							end
						end
					end
				end
			end

	reg [9:0] CH0_address_buf;
	reg [9:0] CH1_address_buf;
/*
	always @(posedge uut.HUFF_clock) begin
	  	// input of antialias
		if (uut.CH0_MAC_RAM_we == 1'b1)
		begin

			if (CH0_ram_data[uut.CH0_MAC_RAM_write_address] != uut.CH0_MAC_RAM_write_data) begin
				$display("L: Data unmatched: addr=%d got=%h expected=%h", uut.CH0_MAC_RAM_write_address, uut.CH0_MAC_RAM_write_data, CH0_ram_data[uut.CH0_MAC_RAM_write_address]);
			end
		end
	end

	always @(posedge uut.HUFF_clock) begin
	  	// input of antialias
		if (uut.CH1_MAC_RAM_we == 1'b1)
		begin

			if (CH1_ram_data[uut.CH1_MAC_RAM_write_address] != uut.CH1_MAC_RAM_write_data) begin
				$display("\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tR: Data unmatched: addr=%d got=%h expected=%h", uut.CH1_MAC_RAM_write_address, uut.CH1_MAC_RAM_write_data, CH1_ram_data[uut.CH1_MAC_RAM_write_address]);
			end
		end
	end
*/
/*
		always @ (posedge uut.MAC_clock) begin
			CH0_address_buf <= uut.MAC_module.CH0_MAC_RAM_ADDRESS_O;
		end

		always @(posedge uut.MAC_clock) begin
		  	// input of antialias
			if ((uut.MAC_module.CH0_state == `STATE_LOAD_HALF_SB) 
			|| (uut.MAC_module.CH0_state == `STATE_ANTIALIAS)
			|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
			|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
			|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
			) 
			begin

	   	$display("ANTI_IN: CH0_state=%d counter=%2d en=%1d ra=%4d do=%4h ed=%h", 
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_counter, 
				uut.MAC_module.CH0_MAC_RAM_EN_O,
				uut.MAC_module.CH0_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH0_MAC_RAM_DATA_I,
				CH0_ram_data[CH0_address_buf]);

			if (CH0_ram_data[CH0_address_buf] != uut.MAC_module.CH0_MAC_RAM_DATA_I) begin
				$display("L: Alias Data unmatched: addr=%d got=%h expected=%h", CH0_address_buf, uut.MAC_module.CH0_MAC_RAM_DATA_I, CH0_ram_data[CH0_address_buf]);
			end

			end
		end


		always @ (posedge uut.MAC_clock) begin
			CH1_address_buf <= uut.MAC_module.CH1_MAC_RAM_ADDRESS_O;
		end

		always @(posedge uut.MAC_clock) begin
		  	// input of antialias
			if ((uut.MAC_module.CH1_state == `STATE_LOAD_HALF_SB) 
			|| (uut.MAC_module.CH1_state == `STATE_ANTIALIAS)
			|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
			|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
			|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
			) 
			begin

	   	$display("ANTI_IN: CH1_state=%d counter=%2d en=%1d ra=%4d do=%4h ed=%h", 
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_counter, 
				uut.MAC_module.CH1_MAC_RAM_EN_O,
				uut.MAC_module.CH1_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH1_MAC_RAM_DATA_I,
				CH1_ram_data[address_buf]);

			if (CH1_ram_data[CH1_address_buf] != uut.MAC_module.CH1_MAC_RAM_DATA_I) begin
				$display("\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tR: Alias Data unmatched: addr=%d got=%h expected=%h", 
				CH1_address_buf, uut.MAC_module.CH1_MAC_RAM_DATA_I, CH1_ram_data[CH1_address_buf]);
			end

			end
		end 
*/
/*
		  always @(posedge uut.MAC_clock) begin
		  	// output of antialias
			if ((uut.MAC_module.CH0_state == `STATE_LOAD_HALF_SB || uut.MAC_module.CH0_state == `STATE_ANTIALIAS) 
			&& uut.MAC_module.CH0_icg_wr_en == 1'b1)
			begin

	   	$display("ANTI_OUT: CH0_state=%d counter=%2d ra1=%4d ra2=%4d di=%4h da=%4h db=%4h mac=%1d wp=%4d we=%1b wa=%4d di=%4h", 
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_counter, 
				uut.MAC_module.CH0_aic_address, 
				uut.MAC_module.CH0_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH0_MAC_RAM_DATA_I,
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_antialias_write_pointer,
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_acc_out);

			end
		end
*/
/*
	 always @(posedge uut.MAC_clock) begin
		  	// output of antialias
			if ((uut.MAC_module.CH1_state == `STATE_LOAD_HALF_SB || uut.MAC_module.CH1_state == `STATE_ANTIALIAS) 
			&& uut.MAC_module.CH1_icg_wr_en == 1'b1)
			begin

	   	$display("ANTI_OUT: CH1_state=%d counter=%2d ra1=%4d ra2=%4d da=%4h db=%4h mac=%1d wp=%4d we=%1b wa=%4d di=%4h", 
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_counter, 
				uut.MAC_module.CH1_aic_address, 
				uut.MAC_module.CH1_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_antialias_write_pointer,
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_acc_out);

			end
		end
*/
/*
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for normal block
		if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE)
		 && (uut.MAC_module.CH0_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				uut.MAC_module.CH0_icg_data_out);
	end
*/
/*
		always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for short block
		if (((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)))
//		 && (uut.MAC_module.CH0_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t addr=%d \t\t en=%d \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH0_MAC_RAM_EN_O,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				uut.MAC_module.CH0_icg_data_out);
	end
*/

/*
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
//		if (uut.MAC_module.CH0_state == `STATE_IMDCT_FREQ_INV)
		if (uut.MAC_module.CH0_icg_wr_en == 1'b1)
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				uut.MAC_module.CH0_icg_data_out);
	end
*/
/*
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
//		if (uut.MAC_module.CH0_state == `STATE_IMDCT_FREQ_INV)
//		if (uut.MAC_module.CH1_icg_wr_en == 1'b1)
		if (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE)
	begin
		mdct_in_fp = $fopen("TXT/mdct_in_temp.txt", "a");
		$fwrite(mdct_in_fp, "RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH1_counter,				
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_icg_mem_en, 
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_in,
				uut.MAC_module.CH1_icg_data_out);
		$fclose(mdct_in_fp);
				 
	   	$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t mo=%h \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH1_counter,				
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.instance_CH1_mult_accum.mult16_module.mult_out,
				uut.MAC_module.CH1_icg_mem_en, 
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_in,
				uut.MAC_module.CH1_icg_data_out);

	end
*/
/*
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// input to MDCT
	if ((uut.MAC_module.CH0_state == `STATE_POLYPHASE_MDCT) && (uut.MAC_module.CH0_counter[9:5] == 0))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_out);
	end
*/
/*
always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// input to MDCT
	if ((uut.MAC_module.CH0_state == `STATE_POLYPHASE_MDCT) && (uut.MAC_module.CH0_counter[9:5] == 0) && (uut.MAC_module.CH1_state == `STATE_WAIT_OTHER_CHANNEL))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_icg_read_pointer,				
				uut.MAC_module.CH1_icg_mem_en, 
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_out);
	end
*/
/*
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of polyphase windowing
	if (uut.MAC_module.CH0_state == `STATE_POLYPHASE_WINDOW 
	&& uut.MAC_module.CH0_PCM_RAM_WE_O)
	begin
	 	$display("LEFT : counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t ca=%d \t\t cd=%h \t\t ra=%d \t\t di=%h \t\t we=%d \t\t wa=%d \t\t wd=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.pwc_address,
				uut.MAC_module.pwc_data_out,
				uut.MAC_module.CH0_psb_address,
				uut.MAC_module.CH0_psb_data_out,
				uut.MAC_module.CH0_PCM_RAM_WE_O,
				uut.MAC_module.CH0_PCM_RAM_ADDRESS_O,
				uut.MAC_module.CH0_PCM_RAM_DATA_O
				);
	end
*/
/*
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of polyphase windowing
	if (uut.MAC_module.CH0_state == `STATE_POLYPHASE_WINDOW)
	begin
		$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t ca=%d \t\t cd=%h \t\t ra=%d \t\t di=%h \t\t we=%d \t\t wa=%d \t\t wd=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.pwc_address,
				uut.MAC_module.pwc_data_out,
				uut.MAC_module.CH1_psb_address,
				uut.MAC_module.CH1_psb_data_out,
				uut.MAC_module.CH1_PCM_RAM_WE_O,
				uut.MAC_module.CH1_PCM_RAM_ADDRESS_O,
				uut.MAC_module.CH1_PCM_RAM_DATA_O
				);
	end
*/
/*
	always @ (uut.MAC_module.state) begin
	if ((uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST) 
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_LOAD_IPG))
		$display("\t");
	end
*/
endmodule

